# coding: utf-8

"""
    SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class FullProfile(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'uuid': 'str',
        'name': 'str',
        'json': 'str',
        'owner': 'str',
        'description': 'str',
        'created': 'datetime',
        'last_edited': 'datetime',
        'authorized': 'bool',
        'editable': 'bool',
        'edit_json': 'str',
        'licenses': 'list[ProfileLicense]'
    }

    attribute_map = {
        'uuid': 'uuid',
        'name': 'name',
        'json': 'json',
        'owner': 'owner',
        'description': 'description',
        'created': 'created',
        'last_edited': 'lastEdited',
        'authorized': 'authorized',
        'editable': 'editable',
        'edit_json': 'editJson',
        'licenses': 'licenses'
    }

    def __init__(self, uuid=None, name=None, json=None, owner=None, description=None, created=None, last_edited=None, authorized=None, editable=None, edit_json=None, licenses=None):  # noqa: E501
        """FullProfile - a model defined in Swagger"""  # noqa: E501
        self._uuid = None
        self._name = None
        self._json = None
        self._owner = None
        self._description = None
        self._created = None
        self._last_edited = None
        self._authorized = None
        self._editable = None
        self._edit_json = None
        self._licenses = None
        self.discriminator = None
        self.uuid = uuid
        self.name = name
        self.json = json
        if owner is not None:
            self.owner = owner
        if description is not None:
            self.description = description
        if created is not None:
            self.created = created
        if last_edited is not None:
            self.last_edited = last_edited
        if authorized is not None:
            self.authorized = authorized
        if editable is not None:
            self.editable = editable
        if edit_json is not None:
            self.edit_json = edit_json
        if licenses is not None:
            self.licenses = licenses

    @property
    def uuid(self):
        """Gets the uuid of this FullProfile.  # noqa: E501

        The profile's ID.  # noqa: E501

        :return: The uuid of this FullProfile.  # noqa: E501
        :rtype: str
        """
        return self._uuid

    @uuid.setter
    def uuid(self, uuid):
        """Sets the uuid of this FullProfile.

        The profile's ID.  # noqa: E501

        :param uuid: The uuid of this FullProfile.  # noqa: E501
        :type: str
        """
        if uuid is None:
            raise ValueError("Invalid value for `uuid`, must not be `None`")  # noqa: E501

        self._uuid = uuid

    @property
    def name(self):
        """Gets the name of this FullProfile.  # noqa: E501

        The profile's user-given name.  # noqa: E501

        :return: The name of this FullProfile.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this FullProfile.

        The profile's user-given name.  # noqa: E501

        :param name: The name of this FullProfile.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def json(self):
        """Gets the json of this FullProfile.  # noqa: E501

        The profile intent itself, stored as a trimmed JSON string.  # noqa: E501

        :return: The json of this FullProfile.  # noqa: E501
        :rtype: str
        """
        return self._json

    @json.setter
    def json(self, json):
        """Sets the json of this FullProfile.

        The profile intent itself, stored as a trimmed JSON string.  # noqa: E501

        :param json: The json of this FullProfile.  # noqa: E501
        :type: str
        """
        if json is None:
            raise ValueError("Invalid value for `json`, must not be `None`")  # noqa: E501

        self._json = json

    @property
    def owner(self):
        """Gets the owner of this FullProfile.  # noqa: E501

        The username of the profile owner.  # noqa: E501

        :return: The owner of this FullProfile.  # noqa: E501
        :rtype: str
        """
        return self._owner

    @owner.setter
    def owner(self, owner):
        """Sets the owner of this FullProfile.

        The username of the profile owner.  # noqa: E501

        :param owner: The owner of this FullProfile.  # noqa: E501
        :type: str
        """

        self._owner = owner

    @property
    def description(self):
        """Gets the description of this FullProfile.  # noqa: E501

        The profile's description.  # noqa: E501

        :return: The description of this FullProfile.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this FullProfile.

        The profile's description.  # noqa: E501

        :param description: The description of this FullProfile.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def created(self):
        """Gets the created of this FullProfile.  # noqa: E501

        The profile's timestamp for creation.  # noqa: E501

        :return: The created of this FullProfile.  # noqa: E501
        :rtype: datetime
        """
        return self._created

    @created.setter
    def created(self, created):
        """Sets the created of this FullProfile.

        The profile's timestamp for creation.  # noqa: E501

        :param created: The created of this FullProfile.  # noqa: E501
        :type: datetime
        """

        self._created = created

    @property
    def last_edited(self):
        """Gets the last_edited of this FullProfile.  # noqa: E501

        The timestamp for when the profile was last edited.  # noqa: E501

        :return: The last_edited of this FullProfile.  # noqa: E501
        :rtype: datetime
        """
        return self._last_edited

    @last_edited.setter
    def last_edited(self, last_edited):
        """Sets the last_edited of this FullProfile.

        The timestamp for when the profile was last edited.  # noqa: E501

        :param last_edited: The last_edited of this FullProfile.  # noqa: E501
        :type: datetime
        """

        self._last_edited = last_edited

    @property
    def authorized(self):
        """Gets the authorized of this FullProfile.  # noqa: E501

        Whether the profile carries an admin's authorization with it.  # noqa: E501

        :return: The authorized of this FullProfile.  # noqa: E501
        :rtype: bool
        """
        return self._authorized

    @authorized.setter
    def authorized(self, authorized):
        """Sets the authorized of this FullProfile.

        Whether the profile carries an admin's authorization with it.  # noqa: E501

        :param authorized: The authorized of this FullProfile.  # noqa: E501
        :type: bool
        """

        self._authorized = authorized

    @property
    def editable(self):
        """Gets the editable of this FullProfile.  # noqa: E501

        Whether the profile can be edited by licensed users.  # noqa: E501

        :return: The editable of this FullProfile.  # noqa: E501
        :rtype: bool
        """
        return self._editable

    @editable.setter
    def editable(self, editable):
        """Sets the editable of this FullProfile.

        Whether the profile can be edited by licensed users.  # noqa: E501

        :param editable: The editable of this FullProfile.  # noqa: E501
        :type: bool
        """

        self._editable = editable

    @property
    def edit_json(self):
        """Gets the edit_json of this FullProfile.  # noqa: E501

        The JSON string for user-configured editable fields.  # noqa: E501

        :return: The edit_json of this FullProfile.  # noqa: E501
        :rtype: str
        """
        return self._edit_json

    @edit_json.setter
    def edit_json(self, edit_json):
        """Sets the edit_json of this FullProfile.

        The JSON string for user-configured editable fields.  # noqa: E501

        :param edit_json: The edit_json of this FullProfile.  # noqa: E501
        :type: str
        """

        self._edit_json = edit_json

    @property
    def licenses(self):
        """Gets the licenses of this FullProfile.  # noqa: E501

        The profile's collection of given licenses.  # noqa: E501

        :return: The licenses of this FullProfile.  # noqa: E501
        :rtype: list[ProfileLicense]
        """
        return self._licenses

    @licenses.setter
    def licenses(self, licenses):
        """Sets the licenses of this FullProfile.

        The profile's collection of given licenses.  # noqa: E501

        :param licenses: The licenses of this FullProfile.  # noqa: E501
        :type: list[ProfileLicense]
        """

        self._licenses = licenses

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(FullProfile, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, FullProfile):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from sense.workflow.provider.provider import Service
from sense.workflow.base.utils import get_logger
from . import sense_utils
from .sense_constants import SERVICE_INSTANCE_KEYS
from .sense_exceptions import SenseException
from sense.workflow.base.config_models import Config
from typing import Union, Dict
from sense.workflow.base.state_models import ServiceState

logger = get_logger()


class SenseService(Service):
    def __init__(self, *, client, label, name: str, profile: str,
                 edit_template: Union[Config, Dict],
                 manifest_template: Union[Config, Dict],
                 saved_state=Union[ServiceState, Dict]):
        super().__init__(label=label, name=name)
        self._client = client
        self.profile = profile

        if isinstance(edit_template, Config):
            self.edit_template: dict = edit_template.attributes
        else:
            self.edit_template: dict = edit_template

        assert isinstance(self.edit_template, dict)

        if isinstance(manifest_template, Config):
            self.manifest_template: dict = manifest_template.attributes
        else:
            self.manifest_template: dict = manifest_template

        assert isinstance(self.manifest_template, dict)

        self.id = str()
        self.state = str()
        self.intents = list()
        self.manifest = dict()

        if isinstance(saved_state, ServiceState):
            self._saved_state: dict = saved_state.attributes
        else:
            self._saved_state: dict = saved_state

    def create(self):
        self.id = sense_utils.find_instance_by_alias(client=self._client, alias=self.name)

        if not self.id:
            logger.debug(f"Creating {self.name}")
            self.id = sense_utils.create_instance(
                client=self._client,
                alias=self.name,
                profile=self.profile,
                edit_template=self.edit_template)

        status = sense_utils.instance_get_status(client=self._client, si_uuid=self.id)
        logger.info(f"Service instance: {self.name} {self.id} with status={status}")

        if 'CREATE - READY' == status:
            return

        self._saved_state = dict()

        if 'INIT' in status:
            status = sense_utils.wait_for_instance_create(client=self._client, si_uuid=self.id)

        if 'FAILED' in status:
            logger.warning(f"Found instance {self.id} with status={status}. Will try to delete")

            try:
                sense_utils.delete_instance(client=self._client, si_uuid=self.id)
                sense_utils.wait_for_delete_instance(client=self._client, si_uuid=self.id, alias=self.name)
            except Exception as e:
                raise SenseException(f"Got exception while deleting instance {self.id} with status={status}:{e}")

        if 'CANCEL - READY' == status:
            logger.info(f"Reprovisioning {self.name}")
            sense_utils.instance_operate(action='reprovision', client=self._client, si_uuid=self.id)
        elif 'CREATE - READY' not in status:
            logger.debug(f"Provisioning {self.name}")
            sense_utils.instance_operate(client=self._client, si_uuid=self.id)

    def wait_for_create(self):
        si_uuid = self.id
        status = sense_utils.wait_for_instance_operate(client=self._client, si_uuid=si_uuid)

        if status not in ['CREATE - READY', 'REINSTATE - READY', 'MODIFY - READY']:
            raise SenseException(f"Creation failed for {si_uuid} {status}")

        logger.debug(f"Retrieving details {self.name} {status}")
        instance_dict = sense_utils.service_instance_details(client=self._client, si_uuid=si_uuid)

        import json

        logger.debug(f"Retrieved details {self.name} {status}: \n{ json.dumps(instance_dict, indent=2)}")

        for key in SERVICE_INSTANCE_KEYS:
            assert key in instance_dict

        assert self.id == instance_dict['referenceUUID']
        self.state = instance_dict['state']
        self.intents = instance_dict['intents']

        if not self.manifest_template:
            return

        self.manifest = self._saved_state.get('manifest', dict())

        if self.manifest:
            logger.info(f"Using saved manifest {self.name}: \n{json.dumps(self.manifest, indent=2)}")
            return

        self.manifest = sense_utils.manifest_create(client=self._client,
                                                    si_uuid=si_uuid, template=self.manifest_template)

        logger.info(f"Retrieved manifest {self.name}: \n{json.dumps(self.manifest, indent=2)}")

        if ('terminals' in self.manifest and 'connections' in self.intents[0]['json']['data']
                and 'terminals' in self.intents[0]['json']['data']['connections'][0]):
            adjusted_terminals = list()
            uris = list()

            for terminal in self.intents[0]['json']['data']['connections'][0]['terminals']:
                uris.append(terminal['uri'])

            for uri in uris:
                for terminal in self.manifest['terminals']:
                    if 'port' in terminal and terminal['port'].startswith(uri + ":"):
                        adjusted_terminals.append(terminal)
                        break

            if adjusted_terminals:
                self.manifest['terminals'] = adjusted_terminals
                logger.info(f"Adjusted terminals in manifest {self.name}: \n{json.dumps(self.manifest, indent=2)}")
            else:
                logger.warning(f"Could not adjust terminals in manifest for {self.name}")

    def delete(self):
        si_uuid = sense_utils.find_instance_by_alias(client=self._client, alias=self.name)

        logger.debug(f"Deleting {self.name} {si_uuid}")

        if si_uuid:
            sense_utils.delete_instance(client=self._client, si_uuid=si_uuid)
            logger.debug(f"Deleted {self.name} {si_uuid}")

    def wait_for_delete(self):
        si_uuid = sense_utils.find_instance_by_alias(client=self._client, alias=self.name)

        logger.debug(f"Deleting {self.name} {si_uuid}")

        if si_uuid:
            sense_utils.wait_for_delete_instance(client=self._client, si_uuid=si_uuid, alias=self.name)
            logger.debug(f"Deleted {self.name} {si_uuid}")



#!/usr/bin/env python3
# coding: utf-8
"""
SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""
from sense.client.requestwrapper import RequestWrapper
from sense.common import classwrapper

@classwrapper
class DiscoverApi():
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """
    def __init__(self, req_wrapper=None):
        if req_wrapper is None:
            self.client = RequestWrapper()
        else:
            self.client = req_wrapper

    def discover_domain_id_get(self, domain_id, **kwargs):  # noqa: E501
        """Edge points discover and description for a specific domain  # noqa: E501

        List all associated edge points (and capabilities)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.discover_domain_id_get(domain_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str domain_id: Name of URI of a target domain (required)
        :return: DomainDescription
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.discover_domain_id_get_with_http_info(
                domain_id, **kwargs)  # noqa: E501
        (data) = self.discover_domain_id_get_with_http_info(
            domain_id, **kwargs)  # noqa: E501
        return data

    def discover_domain_id_get_with_http_info(self, domain_id,
                                              **kwargs):  # noqa: E501
        """Edge points discover and description for a specific domain  # noqa: E501

        List all associated edge points (and capabilities)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.discover_domain_id_get_with_http_info(domain_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str domain_id: Name of URI of a target domain (required)
        :return: DomainDescription
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['domain_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}'"
                                " to method discover_domain_id_get")
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'domain_id' is set
        if ('domain_id' not in params or params['domain_id'] is None):
            raise ValueError(
                "Missing the required parameter `domain_id` when calling `discover_domain_id_get`"
            )  # noqa: E501

        return self.client.request('GET', f'/discover/{domain_id}')

    def discover_domain_id_peers_get(self, domain_id, **kwargs):  # noqa: E501
        """edge points discover and description of peer domain for a given domain or end-site  # noqa: E501

        List peer domain edge points (and capabilities) that connect this domain (by URI or name)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.discover_domain_id_peers_get(domain_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str domain_id: Name of URI of a target end-site domain (required)
        :return: DomainDescription
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.discover_domain_id_peers_get_with_http_info(
                domain_id, **kwargs)  # noqa: E501
        (data) = self.discover_domain_id_peers_get_with_http_info(
            domain_id, **kwargs)  # noqa: E501
        return data

    def discover_domain_id_peers_get_with_http_info(self, domain_id,
                                                    **kwargs):  # noqa: E501
        """edge points discover and description of peer domain for a given domain or end-site  # noqa: E501

        List peer domain edge points (and capabilities) that connect this domain (by URI or name)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.discover_domain_id_peers_get_with_http_info(domain_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str domain_id: Name of URI of a target end-site domain (required)
        :return: DomainDescription
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['domain_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}'"
                                " to method discover_domain_id_peers_get")
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'domain_id' is set
        if ('domain_id' not in params or params['domain_id'] is None):
            raise ValueError(
                "Missing the required parameter `domain_id` when calling `discover_domain_id_peers_get`"
            )  # noqa: E501

        return self.client.request('GET', f'/discover/{domain_id}/peers')

    def discover_domain_id_ipv6pool_get(self, domain_id, **kwargs):  # noqa: E501
        """ discover and description of ipv6 subnet pool for a given domain or end-site  # noqa: E501

        List of ipv6 subnet pools for this domain (by URI or name)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.discover_domain_id_ipv6pool_get(domain_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str domain_id: Name of URI of a target end-site domain (required)
        :return: DomainDescription
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.discover_domain_id_ipv6pool_get_with_http_info(
                domain_id, **kwargs)  # noqa: E501
        (data) = self.discover_domain_id_ipv6pool_get_with_http_info(
            domain_id, **kwargs)  # noqa: E501
        return data

    def discover_domain_id_ipv6pool_get_with_http_info(self, domain_id,
                                                    **kwargs):  # noqa: E501
        """ discover and description of ipv6 subnet pool for a given domain or end-site  # noqa: E501

        List of ipv6 subnet pools for this domain (by URI or name)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.discover_domain_id_ipv6pool_get_with_http_info(domain_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str domain_id: Name of URI of a target end-site domain (required)
        :return: DomainDescription
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['domain_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}'"
                                " to method discover_domain_id_ipv6pool_get")
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'domain_id' is set
        if ('domain_id' not in params or params['domain_id'] is None):
            raise ValueError(
                "Missing the required parameter `domain_id` when calling `discover_domain_id_ipv6pool_get`"
            )  # noqa: E501

        return self.client.request('GET', f'/discover/{domain_id}/ipv6pool')

    def discover_domains_get(self, **kwargs):  # noqa: E501
        """Topology domains  # noqa: E501

        List all known domains (and capabilities?)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.discover_domains_get(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: InlineResponse2001
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.discover_domains_get_with_http_info(
                **kwargs)  # noqa: E501
        (data) = self.discover_domains_get_with_http_info(
            **kwargs)  # noqa: E501
        return data

    def discover_domains_get_with_http_info(self, **kwargs):  # noqa: E501
        """Topology domains  # noqa: E501

        List all known domains (and capabilities?)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.discover_domains_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: InlineResponse2001
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}'"
                                " to method discover_domains_get")
            params[key] = val
        del params['kwargs']
        return self.client.request('GET', '/discover/domains')

    def discover_get(self, **kwargs):  # noqa: E501
        """Topology domains, edge points and peers information  # noqa: E501

        List global domain information  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.discover_get(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: InlineResponse200
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.discover_get_with_http_info(**kwargs)  # noqa: E501
        (data) = self.discover_get_with_http_info(**kwargs)  # noqa: E501
        return data

    def discover_get_with_http_info(self, **kwargs):  # noqa: E501
        """Topology domains, edge points and peers information  # noqa: E501

        List global domain information  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.discover_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: InlineResponse200
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}'"
                                " to method discover_get")
            params[key] = val
        del params['kwargs']
        return self.client.request('GET', '/discover')

    def discover_lookup_name_get(self, name, **kwargs):  # noqa: E501
        """Look up for domain / node / port URI by name  # noqa: E501

        List of URI string  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.discover_lookup_name_get(name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: Resource nml:name or mrs:NetworkAddress or 'fqdn' type (required)
        :param str search: search by name, tag or NetworkAddress .
        :param str type: type of tag or NetworkAddress
        :param bool regex: use full name march or regex
        :return: DomainDescription
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.discover_lookup_name_get_with_http_info(
                name, **kwargs)  # noqa: E501
        (data) = self.discover_lookup_name_get_with_http_info(
            name, **kwargs)  # noqa: E501
        return data

    def discover_lookup_name_get_with_http_info(self, name,
                                                **kwargs):  # noqa: E501
        """Look up for domain / node / port URI by name  # noqa: E501

        List of URI string  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.discover_lookup_name_get_with_http_info(name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str name: Resource nml:name or mrs:NetworkAddress or 'fqdn' type (required)
        :param str search: search by name, tag or NetworkAddress .
        :param str type: type of tag or NetworkAddress
        :param bool regex: use full name march or regex
        :return: DomainDescription
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['name', 'search', 'type', 'regex']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}'"
                                " to method discover_lookup_name_get")
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'name' is set
        if ('name' not in params or params['name'] is None):
            raise ValueError(
                "Missing the required parameter `name` when calling `discover_lookup_name_get`"
            )  # noqa: E501

        query_params = []
        if 'search' in params:
            query_params.append(('search', params['search']))  # noqa: E501
        if 'type' in params:
            query_params.append(('type', params['type']))  # noqa: E501
        if 'regex' in params:
            query_params.append(('regex', params['regex']))  # noqa: E501

        return self.client.request('GET',
                                   f'/discover/lookup/{name}',
                                   query_params=query_params)

    def discover_lookup_rooturi_get(self, uri, **kwargs):  # noqa: E501
        """Look up for domain root URI by given a resource URI  # noqa: E501

        List of URI string  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.discover_lookup_rooturi_get(uri, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str uri: Resource URI (required)
        :return: URI string
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.discover_lookup_rooturi_get_with_http_info(
                uri, **kwargs)  # noqa: E501
        (data) = self.discover_lookup_rooturi_get_with_http_info(
            uri, **kwargs)  # noqa: E501
        return data

    def discover_lookup_rooturi_get_with_http_info(self, uri,
                                                **kwargs):  # noqa: E501
        """Look up for domain root URI by given a resource URI  # noqa: E501

        List of URI string  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.discover_lookup_rooturi_get(uri, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str uri: Resource URI (required)
        :return: URI string
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['uri']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}'"
                                " to method discover_lookup_name_get")
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'name' is set
        if ('uri' not in params or params['uri'] is None):
            raise ValueError(
                "Missing the required parameter `uri` when calling `discover_lookup_rooturi_get`"
            )  # noqa: E501

        query_params = []

        return self.client.request('GET',
                                   f'/discover/lookup/{uri}/rooturi',
                                   query_params=query_params)

    def discover_service_instances_get(self, **kwargs):  # noqa: E501
        """Service discover and description  # noqa: E501

        List service instances  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.discover_service_instances_get(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str search: search by name substring or regex
        :param bool regex: use substring march or regex
        :return: InlineResponse2002
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.discover_service_instances_get_with_http_info(
                **kwargs)  # noqa: E501
        (data) = self.discover_service_instances_get_with_http_info(
            **kwargs)  # noqa: E501
        return data

    def discover_service_instances_get_with_http_info(self,
                                                      **kwargs):  # noqa: E501
        """Service discover and description  # noqa: E501

        List service instances  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.discover_service_instances_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str search: search by name substring or regex
        :param bool regex: use substring march or regex
        :return: InlineResponse2002
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['search', 'regex']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}'"
                                " to method discover_service_instances_get")
            params[key] = val
        del params['kwargs']

        query_params = []
        if 'search' in params:
            query_params.append(('search', params['search']))  # noqa: E501
        if 'regex' in params:
            query_params.append(('regex', params['regex']))  # noqa: E501
        return self.client.request('GET',
                                   '/discover/service/instances',
                                   query_params=query_params)



from types import SimpleNamespace
from typing import List, Tuple

from .exceptions import ParseConfigException
from .config_models import Variable, ProviderConfig, Config, BaseConfig, Dependency, DependencyInfo
from .constants import Constants


class VariableEvaluator:
    def __init__(self, *, variables: List[Variable], providers: List[ProviderConfig], configs: List[Config],
                 resources: List[BaseConfig]):
        self.variables = variables
        self.providers = providers
        self.configs = configs
        self.resources = resources

    def find_variable(self, path: str) -> BaseConfig or Dependency:
        parts = path.split('.')
        assert parts[0] == 'var'

        if len(parts) != 2:
            raise ParseConfigException(f"bad variable dependency {path}")

        for variable in self.variables:
            if variable.name == parts[1]:
                return variable.value

        raise ParseConfigException(f'variable not found at {path}')

    def handle_substitution(self, value):
        if isinstance(value, str):
            value = value.strip()

            if value.startswith('{{') and value.endswith('}}'):
                path = value[2:-2].strip()

                if path.startswith('var'):
                    return self.find_variable(path)

            return value
        elif isinstance(value, list):
            temp = []
            for val in value:
                temp.append(self.handle_substitution(val))
            return temp
        elif isinstance(value, dict):
            temp = {}

            for k, val in value.items():
                temp[k] = self.handle_substitution(val)

            return temp
        elif isinstance(value, SimpleNamespace):
            return self.handle_substitution(vars(value))
        else:
            return value

    def evaluate(self) -> Tuple[List[ProviderConfig], List[Config], List[BaseConfig]]:
        config_entries: List[BaseConfig] = []
        config_entries.extend(self.providers)
        config_entries.extend(self.configs)
        config_entries.extend(self.resources)

        for config_resource_entry in config_entries:
            attrs = {}

            for key, value in config_resource_entry.attributes.items():
                attrs[key] = self.handle_substitution(value)

            config_resource_entry.attributes = attrs

        return self.providers, self.configs, self.resources


class Evaluator:
    def __init__(self, *, providers: List[ProviderConfig], configs: List[Config], resources: List[BaseConfig]):
        self.providers = providers
        self.configs = configs
        self.resources = resources

    def find_object(self, path: str) -> BaseConfig or Dependency:
        parts = path.lower().split('.')
        config_entries: List[BaseConfig] = []
        config_entries.extend(self.providers)
        config_entries.extend(self.configs)
        config_entries.extend(self.resources)

        if len(parts) < 2:
            raise ParseConfigException(f"bad dependency {path}")

        temp = [Config.__name__, ProviderConfig.__name__]

        for config_entry in config_entries:
            if config_entry.type == parts[0] and config_entry.var_name == parts[1]:
                if config_entry.__class__.__name__ not in temp and config_entry.type in Constants.RES_SUPPORTED_TYPES:
                    return DependencyInfo(resource=config_entry, attribute='.'.join(parts[2:]))

                return config_entry

        raise ParseConfigException(f'config entry not found at {path}')

    def handle_substitution(self, value):
        if isinstance(value, str):
            value = value.strip()

            if value.startswith('{{') and value.endswith('}}'):
                path = value[2:-2].strip()
                return self.find_object(path)

            return value
        elif isinstance(value, list):
            temp = []
            for val in value:
                temp.append(self.handle_substitution(val))
            return temp
        elif isinstance(value, dict):
            temp = {}

            for k, val in value.items():
                temp[k] = self.handle_substitution(val)

            return temp
        elif isinstance(value, SimpleNamespace):
            raise Exception(f'{value}')
            # return self.handle_substitution(vars(value))
        else:
            return value

    def evaluate(self) -> Tuple[List[ProviderConfig], List[Config], List[BaseConfig]]:
        for config_resource_entry in self.resources:
            attrs = {}

            for key, value in config_resource_entry.attributes.items():
                attrs[key] = self.handle_substitution(value)

            config_resource_entry.attributes = attrs

        return self.providers, self.configs, self.resources



from typing import List, Union, Dict

from .config_models import ResourceConfig, ProviderConfig
from .parser import Parser
from .constants import Constants


class WorkflowConfig:
    def __init__(self, *, config_dir: str,
                 provider_configs: List[ProviderConfig], resource_configs: List[ResourceConfig]):
        self.provider_configs = provider_configs

        for provider_config in self.provider_configs:
            provider_config.attributes[Constants.CONFIG_DIR] = config_dir

        self.resource_configs = resource_configs

        for resource_config in self.resource_configs:
            resource_config.attributes[Constants.CONFIG_DIR] = config_dir

    def get_provider_configs(self) -> List[ProviderConfig]:
        return self.provider_configs

    def get_resource_configs(self) -> List[ResourceConfig]:
        return self.resource_configs

    @staticmethod
    def parse(*, dir_path: Union[str, None] = None, content: Union[str, None] = None,
              var_dict: Union[Dict, None] = None):
        provider_configs, resource_configs = Parser.parse(dir_path=dir_path, content=content, var_dict=var_dict)
        return WorkflowConfig(config_dir=dir_path, provider_configs=provider_configs, resource_configs=resource_configs)



# coding: utf-8

"""
    SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Connection(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'suggest_ip_range': 'list[ConnectionSuggestIpRange]',
        'schedule': 'ConnectionSchedule',
        'bandwidth': 'ConnectionBandwidth',
        'name': 'str',
        'terminals': 'list[ConnectionTerminals]',
        'path_tprofile': 'ConnectionPathTprofile'
    }

    attribute_map = {
        'suggest_ip_range': 'suggest_ip_range',
        'schedule': 'schedule',
        'bandwidth': 'bandwidth',
        'name': 'name',
        'terminals': 'terminals',
        'path_tprofile': 'path_tprofile'
    }

    def __init__(self, suggest_ip_range=None, schedule=None, bandwidth=None, name=None, terminals=None, path_tprofile=None):  # noqa: E501
        """Connection - a model defined in Swagger"""  # noqa: E501
        self._suggest_ip_range = None
        self._schedule = None
        self._bandwidth = None
        self._name = None
        self._terminals = None
        self._path_tprofile = None
        self.discriminator = None
        if suggest_ip_range is not None:
            self.suggest_ip_range = suggest_ip_range
        if schedule is not None:
            self.schedule = schedule
        if bandwidth is not None:
            self.bandwidth = bandwidth
        self.name = name
        self.terminals = terminals
        if path_tprofile is not None:
            self.path_tprofile = path_tprofile

    @property
    def suggest_ip_range(self):
        """Gets the suggest_ip_range of this Connection.  # noqa: E501

        List of IP Range Suggestions.  # noqa: E501

        :return: The suggest_ip_range of this Connection.  # noqa: E501
        :rtype: list[ConnectionSuggestIpRange]
        """
        return self._suggest_ip_range

    @suggest_ip_range.setter
    def suggest_ip_range(self, suggest_ip_range):
        """Sets the suggest_ip_range of this Connection.

        List of IP Range Suggestions.  # noqa: E501

        :param suggest_ip_range: The suggest_ip_range of this Connection.  # noqa: E501
        :type: list[ConnectionSuggestIpRange]
        """

        self._suggest_ip_range = suggest_ip_range

    @property
    def schedule(self):
        """Gets the schedule of this Connection.  # noqa: E501


        :return: The schedule of this Connection.  # noqa: E501
        :rtype: ConnectionSchedule
        """
        return self._schedule

    @schedule.setter
    def schedule(self, schedule):
        """Sets the schedule of this Connection.


        :param schedule: The schedule of this Connection.  # noqa: E501
        :type: ConnectionSchedule
        """

        self._schedule = schedule

    @property
    def bandwidth(self):
        """Gets the bandwidth of this Connection.  # noqa: E501


        :return: The bandwidth of this Connection.  # noqa: E501
        :rtype: ConnectionBandwidth
        """
        return self._bandwidth

    @bandwidth.setter
    def bandwidth(self, bandwidth):
        """Sets the bandwidth of this Connection.


        :param bandwidth: The bandwidth of this Connection.  # noqa: E501
        :type: ConnectionBandwidth
        """

        self._bandwidth = bandwidth

    @property
    def name(self):
        """Gets the name of this Connection.  # noqa: E501

        Connection name, for labeling purposes.  # noqa: E501

        :return: The name of this Connection.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Connection.

        Connection name, for labeling purposes.  # noqa: E501

        :param name: The name of this Connection.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def terminals(self):
        """Gets the terminals of this Connection.  # noqa: E501

        The array of connection terminals.  # noqa: E501

        :return: The terminals of this Connection.  # noqa: E501
        :rtype: list[ConnectionTerminals]
        """
        return self._terminals

    @terminals.setter
    def terminals(self, terminals):
        """Sets the terminals of this Connection.

        The array of connection terminals.  # noqa: E501

        :param terminals: The terminals of this Connection.  # noqa: E501
        :type: list[ConnectionTerminals]
        """
        if terminals is None:
            raise ValueError("Invalid value for `terminals`, must not be `None`")  # noqa: E501

        self._terminals = terminals

    @property
    def path_tprofile(self):
        """Gets the path_tprofile of this Connection.  # noqa: E501


        :return: The path_tprofile of this Connection.  # noqa: E501
        :rtype: ConnectionPathTprofile
        """
        return self._path_tprofile

    @path_tprofile.setter
    def path_tprofile(self, path_tprofile):
        """Sets the path_tprofile of this Connection.


        :param path_tprofile: The path_tprofile of this Connection.  # noqa: E501
        :type: ConnectionPathTprofile
        """

        self._path_tprofile = path_tprofile

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Connection, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Connection):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ProfileManifest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'name': 'str',
        'description': 'str',
        'editable': 'bool',
        'data': 'str',
        'edit': 'list[ProfileEdit]'
    }

    attribute_map = {
        'name': 'name',
        'description': 'description',
        'editable': 'editable',
        'data': 'data',
        'edit': 'edit'
    }

    def __init__(self, name=None, description=None, editable=None, data=None, edit=None):  # noqa: E501
        """ProfileManifest - a model defined in Swagger"""  # noqa: E501
        self._name = None
        self._description = None
        self._editable = None
        self._data = None
        self._edit = None
        self.discriminator = None
        if name is not None:
            self.name = name
        if description is not None:
            self.description = description
        if editable is not None:
            self.editable = editable
        if data is not None:
            self.data = data
        if edit is not None:
            self.edit = edit

    @property
    def name(self):
        """Gets the name of this ProfileManifest.  # noqa: E501

        The profile name.  # noqa: E501

        :return: The name of this ProfileManifest.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ProfileManifest.

        The profile name.  # noqa: E501

        :param name: The name of this ProfileManifest.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def description(self):
        """Gets the description of this ProfileManifest.  # noqa: E501

        The profile description.  # noqa: E501

        :return: The description of this ProfileManifest.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ProfileManifest.

        The profile description.  # noqa: E501

        :param description: The description of this ProfileManifest.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def editable(self):
        """Gets the editable of this ProfileManifest.  # noqa: E501

        Whether the profile is editable by licensed users.  # noqa: E501

        :return: The editable of this ProfileManifest.  # noqa: E501
        :rtype: bool
        """
        return self._editable

    @editable.setter
    def editable(self, editable):
        """Sets the editable of this ProfileManifest.

        Whether the profile is editable by licensed users.  # noqa: E501

        :param editable: The editable of this ProfileManifest.  # noqa: E501
        :type: bool
        """

        self._editable = editable

    @property
    def data(self):
        """Gets the data of this ProfileManifest.  # noqa: E501

        Profile JSON in string format.  # noqa: E501

        :return: The data of this ProfileManifest.  # noqa: E501
        :rtype: str
        """
        return self._data

    @data.setter
    def data(self, data):
        """Sets the data of this ProfileManifest.

        Profile JSON in string format.  # noqa: E501

        :param data: The data of this ProfileManifest.  # noqa: E501
        :type: str
        """

        self._data = data

    @property
    def edit(self):
        """Gets the edit of this ProfileManifest.  # noqa: E501

        The profile's editable field configuration.  # noqa: E501

        :return: The edit of this ProfileManifest.  # noqa: E501
        :rtype: list[ProfileEdit]
        """
        return self._edit

    @edit.setter
    def edit(self, edit):
        """Sets the edit of this ProfileManifest.

        The profile's editable field configuration.  # noqa: E501

        :param edit: The edit of this ProfileManifest.  # noqa: E501
        :type: list[ProfileEdit]
        """

        self._edit = edit

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ProfileManifest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ProfileManifest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



class Constants:
    PROVIDER_STATE = 'provider_state'
    LABELS = 'labels'

    RES_TYPE = 'res_type'
    CONFIG = 'config'
    RES_NAME_PREFIX = "name_prefix"
    CREDENTIAL_FILE = 'credential_file'
    PROFILE = "profile"
    CONFIG_DIR = "config_dir"

    POOL = 'pool'
    ADDR_TYPE = "addr_type"
    NET_MASK = 'netmask'

    SENSE_EXTENSION = "sense"
    PROVIDER = 'provider'
    LABEL = 'label'
    RES_COUNT = 'count'

    PROVIDER_CLASSES = {
        "sense": "sense.workflow.sense.sense_provider.SenseProvider"
    }

    EDIT_TEMPLATE = 'edit_template'
    CONFIG_SUPPORTED_TYPES = [EDIT_TEMPLATE, 'manifest_template']
    RES_TYPE_SERVICE = "service"
    RES_SUPPORTED_TYPES = [RES_TYPE_SERVICE]
    RES_RESTRICTED_TYPES = []

    EXTERNAL_DEPENDENCIES = "external_dependencies"
    RESOLVED_EXTERNAL_DEPENDENCIES = "resolved_external_dependencies"

    INTERNAL_DEPENDENCIES = "internal_dependencies"
    RESOLVED_INTERNAL_DEPENDENCIES = "resolved_internal_dependencies"

    SAVED_STATES = "saved_states"
    RES_CREATION_DETAILS = "creation_details"
    EXTERNAL_DEPENDENCY_STATES = "external_dependency_states"

    LOGGING = "logging"
    PROPERTY_CONF_LOG_FILE = 'log-file'
    PROPERTY_CONF_LOG_LEVEL = 'log-level'
    PROPERTY_CONF_LOG_RETAIN = 'log-retain'
    PROPERTY_CONF_LOG_SIZE = 'log-size'
    PROPERTY_CONF_LOGGER = "logger"



#!/usr/bin/env python3
# coding: utf-8
"""
    SENSE-SiteRM Services API
"""
from sense.client.siterm.requestwrapper import RequestWrapper


class ServicesApi:
    """Services API for SENSE-SiteRM"""
    def __init__(self):
        self.client = RequestWrapper()

    def _getSitename(self, **kwargs):
        sitename = kwargs.get("sitename", None)
        if not sitename:
            raise Exception("Sitename is required for get_configuration")
        return sitename

    def get_servicestates(self, **kwargs):
        """Get service states from the SENSE-SiteRM API"""
        sitename = self._getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/{sitename}/sitefe/json/frontend/getservicestates",
                                       **{"verb": "GET", "data": {}})

    def reloadconfig(self, **kwargs):
        """Reload configuration via SENSE-SiteRM API for specific entry"""
        vals = {"hostname": None, "servicename": "ALL",
                "sitename": None, "action": "reload"}
        for key in vals:
            if key in kwargs:
                vals[key] = kwargs[key]
            if not vals[key]:
                raise Exception(f"Key {key} is required for reloadconfig")
        sitename = self._getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/{sitename}/sitefe/json/frontend/serviceaction",
                                       **{"verb": "POST", "data": vals})



from sense.client.intent_api import IntentApi
from sense.client.profile_api import ProfileApi
from sense.common import loadJSON
import unittest
import pytest
import re
import json
import time
import sys

# Append root for proper imports
sys.path.append('')
#

from sense.client.workflow_combined_api import WorkflowCombinedApi
from sense.client.workflow_phased_api import WorkflowPhasedApi
from sense.models.service_intent import ServiceIntent


class TestCombinedWorkflow(unittest.TestCase):
    def setUp(self) -> None:
        self.client = WorkflowCombinedApi()

    def tearDown(self) -> None:
        if self.profile_uuid is not None:
            ProfileApi().profile_delete(self.profile_uuid)
        if self.client.si_uuid is not None:
            status = self.client.instance_get_status()
            if 'CANCEL - READY' in status or 'CANCEL - COMMITTED' in status or 'CREATE - COMPILED' in status:
                self.client.instance_delete()
            elif self.client.si_uuid in status and 'not found' in status:
                print(
                    f'Warning! service instance "{self.client.si_uuid}" no longer exists.'
                )
            else:
                print(
                    f'Warning! service instance "{self.client.si_uuid}" remains.'
                )

    @unittest.skip("skipping")
    def test_create_and_delete(self):
        # new instance UUID
        self.client.instance_new()
        assert re.match(
            '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}',
            self.client.si_uuid)
        # create instance with intent
        intent_file = open("test/requests/request-1.json")
        intent = json.load(intent_file)
        intent_file.close()
        response = self.client.instance_create(json.dumps(intent))
        assert self.client.si_uuid in response
        print(f'created with intent: {response}')
        # delete instance with intent
        response = self.client.instance_delete()

    def test_workflow(self):
        # new instance UUID
        self.client.instance_new()
        assert re.match(
            '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}',
            self.client.si_uuid)

        # create instance with intent
        intent_file = open("test/requests/request-1.json")
        intent = json.load(intent_file)
        intent['alias'] = f'{intent["alias"]}-{self.client.si_uuid}'
        intent_file.close()
        response = self.client.instance_create(json.dumps(intent))
        assert self.client.si_uuid in response
        print(f'created with intent: {response}')

        # combined workflow does not support propagate etc. actions
        with self.assertRaises(ValueError):
            self.client.instance_operate('propagate', sync='true')

        # provision in sync mode
        self.client.instance_operate('provision', sync='true')
        status = self.client.instance_get_status()
        print(f'provision status={status}')
        assert 'CREATE - READY' in status

        # cancel in sync mode
        self.client.instance_operate('cancel', sync='true')
        status = self.client.instance_get_status()
        print(f'cancel status={status}')
        assert 'CANCEL - READY' in status

        # delete instance with intent
        self.client.instance_delete()
        response = self.client.instance_get_status()
        assert self.client.si_uuid in response and 'not found' in response

    def test_intent_versioning(self):
        # new instance UUID
        self.client.instance_new()
        assert re.match(
            '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}',
            self.client.si_uuid)

        # create instance with intent
        intent_file = open("test/requests/request-1.json")
        intent = json.load(intent_file)
        intent['alias'] = f'{intent["alias"]}-{self.client.si_uuid}'
        intent_file.close()
        response = self.client.instance_create(json.dumps(intent))
        assert self.client.si_uuid in response
        print(f'created with intent: {response}')

        intent_file = open("test/requests/request-2.json")
        intent2 = json.load(intent_file)
        intent_file.close()
        response = self.client.instance_create(json.dumps(intent2))
        assert self.client.si_uuid in response
        print(f'created with intent: {response}')

        response = self.client.instance_get_intents()
        print(response)
        intent_list = json.loads(response)
        assert len(intent_list) == 2

        intent_1st_uuid = intent_list[0]['id']
        print(f'intent_1st_uuid={intent_1st_uuid}')
        intent_2nd_uuid = intent_list[1]['id']
        print(f'intent_2nd_uuid={intent_2nd_uuid}')

        # provision with a given intent
        self.client.instance_operate("provision",
                                     sync='true',
                                     intent=intent_1st_uuid)
        status = self.client.instance_get_status()
        print(f'provision status={status}')
        assert 'CREATE - READY' in status

        # cancel in sync mode
        self.client.instance_operate('cancel', sync='true')
        status = self.client.instance_get_status()
        print(f'cancel status={status}')
        assert 'CANCEL - READY' in status

        # reprovision with another given intent
        self.client.instance_operate("reprovision",
                                     sync='true',
                                     intent=intent_2nd_uuid)
        status = self.client.instance_get_status()
        print(f'provision status={status}')
        assert 'REINSTATE - READY' in status

        # cancel in sync mode
        self.client.instance_operate('cancel', sync='true')
        status = self.client.instance_get_status()
        print(f'cancel status={status}')
        assert 'CANCEL - READY' in status


class TestPhasedWorkflow(unittest.TestCase):
    def setUp(self) -> None:
        self.client = WorkflowPhasedApi()

    def tearDown(self) -> None:
        if self.client.si_uuid is not None:
            status = self.client.instance_get_status()
            if 'CANCEL - READY' in status or 'CANCEL - COMMITTED' in status or 'CREATE - COMPILED' in status:
                self.client.instance_delete()
            elif self.client.si_uuid in status and 'not found' in status:
                print(
                    f'Warning! service instance "{self.client.si_uuid}" no longer exists.'
                )
            else:
                print(
                    f'Warning! service instance "{self.client.si_uuid}" remains.'
                )

    def test_workflow(self):
        # new instance UUID
        self.client.instance_new()
        assert re.match(
            '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}',
            self.client.si_uuid)

        # create instance with intent
        intent_file = open("test/requests/request-1.json")
        intent = json.load(intent_file)
        intent['alias'] = f'{intent["alias"]}-{self.client.si_uuid}'
        intent_file.close()
        response = self.client.instance_create(json.dumps(intent))
        assert self.client.si_uuid in response
        print(f'created with intent: {response}')

        # phased workflow does not support provision etc. actions
        with self.assertRaises(ValueError):
            self.client.instance_operate('provision', sync='true')

        # propagate action in sync mode
        self.client.instance_operate('propagate', sync='true')
        status = self.client.instance_get_status()
        print(f'propagate status={status}')
        assert 'CREATE - PROPAGATED' in status

        # commit in async mode
        self.client.instance_operate('commit', sync='false')
        status = self.client.instance_get_status()
        print(f'commit status={status}')
        assert 'CREATE - COMMITTING' in status

        # waiting for COMMITTED or FAILED
        commit_timeout = 300  # seconds
        poll_interval = 30
        while 'COMMITTING' in status and commit_timeout > 0:
            time.sleep(poll_interval)
            status = self.client.instance_get_status()
            commit_timeout -= poll_interval
        print(f'commit status={status}')

        assert 'CREATE - COMMITTED' in status

        self.client.instance_operate('verify', sync='false')
        verify_timeout = 600  # seconds
        while 'COMMITTED' in status:
            time.sleep(poll_interval)
            status = self.client.instance_get_status()
            verify_timeout -= poll_interval
        print(f'commit status={status}')
        assert 'CREATE - READY' in status

        # release, propagate, commit and verify
        self.client.instance_operate('release', sync='true')
        status = self.client.instance_get_status()
        print(f'release status={status}')
        assert 'CANCEL - PROPAGATED' in status

        # commit in async mode
        self.client.instance_operate('commit', sync='false')
        status = self.client.instance_get_status()
        print(f'commit status={status}')
        assert 'CANCEL - COMMITTING' in status

        # waiting for COMMITTED or FAILED
        commit_timeout = 300  # seconds
        poll_interval = 30
        while 'COMMITTING' in status and commit_timeout > 0:
            time.sleep(poll_interval)
            status = self.client.instance_get_status()
            commit_timeout -= poll_interval
        print(f'commit status={status}')
        assert 'CANCEL - COMMITTED' in status

        self.client.instance_operate('verify', sync='false')
        verify_timeout = 600  # seconds
        while 'COMMITTED' in status:
            time.sleep(poll_interval)
            status = self.client.instance_get_status()
            verify_timeout -= poll_interval
        print(f'commit status={status}')
        assert 'CANCEL - READY' in status

        self.client.instance_delete()
        response = self.client.instance_get_status()
        assert self.client.si_uuid in response and 'not found' in response

    # def OLD_using_profile(self):
    #     PROFILE_ID = '2fd19216-67e3-4eac-bc70-ac7d98e05cb1'
    #     # TODO: place assert params in config files
    #     profile_list = self.client.profile_list()
    #     print(profile_list)
    #     assert PROFILE_ID in str(profile_list)

    #     profile_data = self.client.profile_describe(PROFILE_ID)
    #     print(profile_data)
    #     # assert 'MAC-DNC-1' in str(profile_data)

    #     # new instance UUID
    #     self.client.instance_new()
    #     assert re.match(
    #         '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}',
    #         self.client.si_uuid)
    #     intent = ServiceIntent(service='dnc',
    #                            profileID=PROFILE_ID,
    #                            alias='DNC-profile-' + PROFILE_ID)
    #     response = self.client.instance_create(json.dumps(intent.to_dict()))
    #     assert self.client.si_uuid in response
    #     print(f'created with intent: {response}')

    #     pass

    def test_using_profile(self):
        # Create profile for testing
        api = ProfileApi()
        profile = loadJSON("requests/profile-1.json")
        self.profile_uuid = api.profile_create(json.dumps(profile))
        #
        # Test creation with only profile ID
        self.client.instance_new()
        assert re.match(
            '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}',
            self.client.si_uuid)
        intent = loadJSON("requests/request-profile.json")
        intent["profileID"] = self.profile_uuid
        response = self.client.instance_create(json.dumps(intent))
        # print(response)
        assert self.client.si_uuid in response
        self.client.instance_delete()
        self.client.si_uuid = None

        #
        # Test editable field overwrite
        uri_1 = "urn:ogf:network:maxgigapop.net:2013:180-134.research.maxgigapop.net"
        uri_2 = "urn:ogf:network:maxgigapop.net:2013:s0:1_1:wash-cr5"
        path_1 = profile["edit"][0]["path"]
        path_2 = profile["edit"][1]["path"]
        query = dict([("ask", "edit"),
                      ("options", [dict([(path_1, uri_1), (path_2, uri_2)])])])
        intent["queries"] = [query]
        print(intent)

        self.client.instance_new()
        response = self.client.instance_create(json.dumps(intent))

        status = self.client.instance_get_status()
        print(f'create status={status}')
        assert 'CREATE - COMPILED' in status

        intentAPI = IntentApi()
        intentAPI.si_uuid = self.client.si_uuid
        intents = intentAPI.instance_get_intents()
        assert self.client.si_uuid in response
        assert uri_1 in intents
        assert uri_2 in intents
        self.client.instance_delete()
        self.client.si_uuid = None

        # TODO: no permission for profiles by other user

        # Clean-up single use profile
        api.profile_delete(self.profile_uuid)
        self.profile_uuid = None


if __name__ == '__main__':
    unittest.main()



class ParseConfigException(Exception):
    pass


class ResourceTypeNotSupported(Exception):
    pass


class ProviderTypeNotSupported(Exception):
    pass


class ConfigTypeNotSupported(Exception):
    pass


class StateException(Exception):
    pass


class ControllerException(Exception):
    pass


class ProviderException(Exception):
    pass



import unittest
import pytest
import re
import json
import time
import sys

# Append root for proper imports
sys.path.append('')
#

from sense.client.instance_api import InstanceApi
from sense.models.service_intent import ServiceIntent


class TestInstanceApi(unittest.TestCase):
    def setUp(self) -> None:
        self.client = InstanceApi()

    def tearDown(self) -> None:
        if self.client.si_uuid is not None:
            status = self.client.instance_get_status()
            if 'CANCEL - READY' in status or 'CANCEL - COMMITTED' in status or 'CREATE - COMPILED' in status:
                self.client.instance_delete()
            elif self.client.si_uuid in status and 'not found' in status:
                print(
                    f'Warning! service instance "{self.client.si_uuid}" no longer exists.'
                )
            else:
                print(
                    f'Warning! service instance "{self.client.si_uuid}" remains.'
                )

    def test_logging(self):
        # new instance UUID
        self.client.instance_new()
        assert re.match(
            '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}',
            self.client.si_uuid)
        # create instance with intent
        intent_file = open("test/requests/request-1.json")
        intent = json.load(intent_file)
        intent_file.close()
        response = self.client.instance_create(json.dumps(intent))
        assert self.client.si_uuid in response
        print(f'created with intent: {response}')

        logging = self.client.instance_get_logging()
        print(logging)

        # delete instance with intent
        response = self.client.instance_delete()
        assert self.client.si_uuid in response and 'not found' in response

    def test_modify_add_connection(self):
        # create and provision instance
        # new instance UUID
        self.client.instance_new()
        assert re.match(
            '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}',
            self.client.si_uuid)
        # create instance with intent
        intent_file = open("test/requests/request-1.json")
        intent = json.load(intent_file)
        intent['alias'] = f'{intent["alias"]}-{self.client.si_uuid}'
        intent_file.close()
        response = self.client.instance_create(json.dumps(intent))
        assert self.client.si_uuid in response
        print(f'created with intent: {response}')
        # provision with a given intent
        self.client.instance_operate("provision", sync='true')
        status = self.client.instance_get_status()
        print(f'provision status={status}')
        assert 'CREATE - READY' in status
        # modify testing
        # add connection
        intent_file = open("test/requests/request-1-2.json")
        intent = json.load(intent_file)
        intent['alias'] = f'{intent["alias"]}-{self.client.si_uuid}'
        intent_file.close()
        response = self.client.instance_modify(json.dumps(intent), sync="true")
        status = self.client.instance_get_status()
        print(f'modify status={status}')
        assert 'MODIFY - READY' in status

        # cancel in sync mode
        self.client.instance_operate('cancel', sync='true')
        status = self.client.instance_get_status()
        print(f'cancel status={status}')
        assert 'CANCEL - READY' in status

        # delete instance with intent
        self.client.instance_delete()
        response = self.client.instance_get_status()
        assert self.client.si_uuid in response and 'not found' in response

    def test_modify_ip_address(self):
        # TODO: create and provision instance
        # TODO: test modify IP
        # TODO: test modify add / remove connections
        pass


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
"""Common functions for SENSE-0 API Client"""
import os
import inspect
import time
import json
from yaml import safe_load as yload

def functionwrapper(func):
    """Function wrapper to print start/runtime/end"""
    def wrapper(*args, **kwargs):
        if isDebugSet():
            print(f"[WRAPPER][{time.time()}] Enter {func.__qualname__}, {func.__code__.co_filename}")
            start_time = time.perf_counter()
            result = func(*args, **kwargs)
            end_time = time.perf_counter()
            total_time = end_time - start_time
            print(f"[WRAPPER][{time.time()}] Function {func.__qualname__} {args} {kwargs} Took {total_time:.4f} seconds")
            print(f"[WRAPPER][{time.time()}] Leave {func.__qualname__}")
        else:
            result = func(*args, **kwargs)
        return result

    return wrapper

def classwrapper(cls):
    """Class wrapper to print all functions start/runtime/end"""
    for name, method in cls.__dict__.items():
        if callable(method) and name != "__init__":
            if inspect.isfunction(method):
                if inspect.signature(method).parameters.get('self'):
                    setattr(cls, name, functionwrapper(method))
            elif inspect.ismethod(method):
                if inspect.signature(method).parameters:
                    firstParam = next(iter(inspect.signature(method).parameters))
                    if firstParam == 'self':
                        setattr(cls, name, functionwrapper(method))
    return cls

def loadJSON(path):
    """Load JSON file"""
    ret = None
    with open(path, 'r', encoding="utf-8") as fd:
        ret = json.load(fd)
    return ret

def isDebugSet():
    """Check if debug is set"""
    if os.environ.get('SENSE_FULL_DEBUG'):
        return True
    return False

def loadYamlFile(fname):
    """Load Yaml file"""
    try:
        with open(fname, 'r', encoding='utf-8') as fd:
            return yload(fd.read())
    except Exception as e:
        print(f"Error loading yaml file: {fname}. Error: {e}")
    return {}

def getConfig(configFile='/etc/sense-o-auth.yaml'):
    """Get SENSE Auth config file"""
    if os.getenv('SENSE_AUTH_OVERRIDE'):
        configFile = os.getenv('SENSE_AUTH_OVERRIDE')
        if not os.path.isfile(configFile):
            raise Exception(f"SENSE_AUTH_OVERRIDE env flag set, but file not found: {configFile}")
    elif not os.path.isfile(configFile):
        configFile = os.getenv('HOME') + '/.sense-o-auth.yaml'
        if not os.path.isfile(configFile):
            raise Exception(f"Config file not found: {configFile}")
    return loadYamlFile(configFile)

def getHTTPTimeout():
    """Get HTTP Timeout from env or default to 300 seconds"""
    return int(os.environ.get('SENSE_TIMEOUT', 300))

def getHTTPRetries():
    """Get HTTP Retries from env or default to 1"""
    return int(os.environ.get('SENSE_RETRIES', 1))



# coding: utf-8

"""
    SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ServiceIntent(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'service': 'str',
        'options': 'list[str]',
        'profile_id': 'str',
        'queries': 'list[ServiceIntentQueries]',
        'data': 'OneOfServiceIntentData',
        'alias': 'str'
    }

    attribute_map = {
        'service': 'service',
        'options': 'options',
        'profile_id': 'profileID',
        'queries': 'queries',
        'data': 'data',
        'alias': 'alias'
    }

    def __init__(self, service=None, options=None, profile_id=None, queries=None, data=None, alias=None):  # noqa: E501
        """ServiceIntent - a model defined in Swagger"""  # noqa: E501
        self._service = None
        self._options = None
        self._profile_id = None
        self._queries = None
        self._data = None
        self._alias = None
        self.discriminator = None
        self.service = service
        if options is not None:
            self.options = options
        if profile_id is not None:
            self.profile_id = profile_id
        if queries is not None:
            self.queries = queries
        if data is not None:
            self.data = data
        self.alias = alias

    @property
    def service(self):
        """Gets the service of this ServiceIntent.  # noqa: E501

        The type of service being created.  # noqa: E501

        :return: The service of this ServiceIntent.  # noqa: E501
        :rtype: str
        """
        return self._service

    @service.setter
    def service(self, service):
        """Sets the service of this ServiceIntent.

        The type of service being created.  # noqa: E501

        :param service: The service of this ServiceIntent.  # noqa: E501
        :type: str
        """
        if service is None:
            raise ValueError("Invalid value for `service`, must not be `None`")  # noqa: E501

        self._service = service

    @property
    def options(self):
        """Gets the options of this ServiceIntent.  # noqa: E501

        Array of option flags.  # noqa: E501

        :return: The options of this ServiceIntent.  # noqa: E501
        :rtype: list[str]
        """
        return self._options

    @options.setter
    def options(self, options):
        """Sets the options of this ServiceIntent.

        Array of option flags.  # noqa: E501

        :param options: The options of this ServiceIntent.  # noqa: E501
        :type: list[str]
        """

        self._options = options

    @property
    def profile_id(self):
        """Gets the profile_id of this ServiceIntent.  # noqa: E501

        Optional link to a profile via UUID.  # noqa: E501

        :return: The profile_id of this ServiceIntent.  # noqa: E501
        :rtype: str
        """
        return self._profile_id

    @profile_id.setter
    def profile_id(self, profile_id):
        """Sets the profile_id of this ServiceIntent.

        Optional link to a profile via UUID.  # noqa: E501

        :param profile_id: The profile_id of this ServiceIntent.  # noqa: E501
        :type: str
        """

        self._profile_id = profile_id

    @property
    def queries(self):
        """Gets the queries of this ServiceIntent.  # noqa: E501


        :return: The queries of this ServiceIntent.  # noqa: E501
        :rtype: list[ServiceIntentQueries]
        """
        return self._queries

    @queries.setter
    def queries(self, queries):
        """Sets the queries of this ServiceIntent.


        :param queries: The queries of this ServiceIntent.  # noqa: E501
        :type: list[ServiceIntentQueries]
        """

        self._queries = queries

    @property
    def data(self):
        """Gets the data of this ServiceIntent.  # noqa: E501


        :return: The data of this ServiceIntent.  # noqa: E501
        :rtype: OneOfServiceIntentData
        """
        return self._data

    @data.setter
    def data(self, data):
        """Sets the data of this ServiceIntent.


        :param data: The data of this ServiceIntent.  # noqa: E501
        :type: OneOfServiceIntentData
        """

        self._data = data

    @property
    def alias(self):
        """Gets the alias of this ServiceIntent.  # noqa: E501

        Alias to give to the created service instance.  # noqa: E501

        :return: The alias of this ServiceIntent.  # noqa: E501
        :rtype: str
        """
        return self._alias

    @alias.setter
    def alias(self, alias):
        """Sets the alias of this ServiceIntent.

        Alias to give to the created service instance.  # noqa: E501

        :param alias: The alias of this ServiceIntent.  # noqa: E501
        :type: str
        """
        if alias is None:
            raise ValueError("Invalid value for `alias`, must not be `None`")  # noqa: E501

        self._alias = alias

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ServiceIntent, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ServiceIntent):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import os
import unittest
import pytest
import re
import json
import string
import random
import sys

# Append root for proper imports
sys.path.append('')
#

#
from sense.client.logging_api import LoggingApi
from sense.common import loadJSON
#


class TestLoggingApi(unittest.TestCase):
    def setUp(self) -> None:
        self.client = LoggingApi()

    def tearDown(self) -> None:
        pass

    def test_get_config(self):
        #
        # TESTING: GET /logging/config
        conf = json.loads(self.client.logging_get_config())
        assert conf.get("loggers").get("console")

    def test_set_filter(self):
        # PRE-OP: get current filter
        testString = ''.join(
            random.choice(string.ascii_lowercase) for i in range(10))
        pre_conf = json.loads(self.client.logging_get_config())
        if "filter" in pre_conf:
            pre = re.findall("pattern=(.*)", pre_conf.get("filter"))
        else:
            pre = None

        #
        # TESTING: PUT /logging/filter
        self.client.logging_set_filter(body=testString)
        conf = json.loads(self.client.logging_get_config())
        assert testString in conf.get("filter")

        # POST-OP: set old filter
        if pre and len(pre) > 0:
            self.client.logging_set_filter(body=pre[0])
        else:
            self.client.logging_set_filter()

    def test_put_level(self):
        # PRE-OP: get current console level
        pre_conf = json.loads(self.client.logging_get_config())
        pre_level = pre_conf.get("loggers").get("console")

        #
        # TESTING: PUT /logging/config/{logger}/{level}
        self.client.logging_set_logger_level("console", "ERROR")
        conf = json.loads(self.client.logging_get_config())
        assert conf.get("loggers").get("console") == "ERROR"

        # POST-OP: set old level
        if pre_level:
            self.client.logging_set_logger_level("console", pre_level)
        else:
            self.client.logging_set_logger_level("console", "INFO")

    def test_get_logs(self):
        #
        # TESTING: GET /logging/logs/{siUUID}
        # -- 1: full logs
        self.client.si_uuid = "null"
        res = json.loads(self.client.instance_get_logging())
        assert "data" in res and len(res.get("data")) > 0
        # -- 2: instance logs (if instances present)
        filtered = list(
            filter(lambda log: log.get("referenceUUID"), res.get("data")))
        if len(filtered) > 0:
            self.client.si_uuid = filtered[0].get("referenceUUID")
            filtered_res = json.loads(self.client.instance_get_logging())
            assert_filtered = list(
                filter(
                    lambda log: log.get("referenceUUID") != self.client.
                    si_uuid, filtered_res.get("data")))
            assert len(assert_filtered) == 0

    # TODO: implement safe archival test.
    def TODO_archive_logs(self):
        # TESTING: PUT /logging/archive/{days}
        pass


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# pylint: disable=R1705
# coding: utf-8
"""
    SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation

    OpenAPI spec version: 2.0.2

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""
from sense.client.requestwrapper import RequestWrapper
from sense.common import classwrapper

@classwrapper
class WorkflowCombinedApi():
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, req_wrapper=None):
        if req_wrapper is None:
            self.client = RequestWrapper()
        else:
            self.client = req_wrapper
        self.si_uuid = None

        self._allowed_actions = ['provision', 'cancel', 'reprovision']

    def instance_new(self, **kwargs):  # noqa: E501
        """Generate new service instance UUID  # noqa: E501

        Retrieves UUID for new instance from backend.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_new(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            data = self.instance_get_with_http_info(**kwargs)
            self.si_uuid = data
            return data  # noqa: E501
        else:
            (data) = self.instance_get_with_http_info(**kwargs)  # noqa: E501
            self.si_uuid = data
            return data

    def instance_get_with_http_info(self, **kwargs):  # noqa: E501
        """Generate new service instance UUID  # noqa: E501

        Retrieves UUID for new instance from backend.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}'"
                                 " to method instance_get")
            params[key] = val
        del params['kwargs']

        return self.client.request('GET', '/instance')

    # TODO: check action types and only allow for combined ops
    def instance_operate(self, action, **kwargs):  # noqa: E501
        """Operate on a service instance  # noqa: E501

        Request an operation for specified service instance.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_operate(si_uuid, action, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: service instance UUID (required)
        :param Operation action: Service operation requested
            * `provision` - Composite service to propagate, commit and verify instance.
            * `cancel` - Composite service to cancel, propagate, commit and verify instance.
            * `reprovision` - Composite service to reinstate, propagate, commit and verify instance.
        :param bool sync: Whether to perform the operation in a synchronous/blocking mode.
        :param bool force: Whether to force the operation on a failed instance.
        :param str intent: Intent UUID string or 'last'.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid and not kwargs.get('si_uuid'):
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid']:
            raise ValueError("Missing the required parameter `si_uuid`")

        if action not in self._allowed_actions:
            raise ValueError(
                f'operate action must of one of {self._allowed_actions}')


        if kwargs.get('async_req'):
            return self.instance_si_uuid_action_put_with_http_info(
                action, **kwargs)  # noqa: E501
        else:
            (data) = self.instance_si_uuid_action_put_with_http_info(
                action, **kwargs)  # noqa: E501
            return data

    def instance_si_uuid_action_put_with_http_info(self, action, **kwargs):
        """Operate on a service instance

        Request an operation for specified service instance.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_si_uuid_action_put_with_http_info(si_uuid, action, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: service instance UUID (required)
        :param Operation action: Service operation requested
          * `cancel` - Composite service to cancel, propagate, commit and verify instance.
          * `release` - Composite service to cancel and propagate in 2-phase commit process.
          * `verify` - Begin service verification process. (required)
        :param bool sync: Whether to perform the operation in a synchronous/blocking mode.
        :param bool force: Whether to force the operation on a failed instance.
        :param str intent: Intent UUID string or 'last'.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['si_uuid', 'action', 'sync', 'force', 'intent']
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}'"
                                " to method instance_si_uuid_action_put")
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `instance_si_uuid_action_put`"
            )  # noqa: E501
        # verify the required parameter 'action' is set
        if ('action' not in params or params['action'] is None):
            raise ValueError(
                "Missing the required parameter `action` when calling `instance_si_uuid_action_put`"
            )  # noqa: E501

        query_params = []
        if 'sync' in params:
            query_params.append(('sync', params['sync']))
        if 'force' in params:
            query_params.append(('force', params['force']))
        if 'intent' in params:
            query_params.append(('intent', params['intent']))

        return self.client.request('PUT',
                                   '/instance/' + kwargs['si_uuid'] + '/' + action,
                                   query_params=query_params)

    def instance_delete(self, **kwargs):  # noqa: E501
        """Delete a service instance  # noqa: E501

        Deletes the specified service instance.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_delete(si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: service instance UUID (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid and not kwargs.get('si_uuid'):
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid']:
            raise ValueError("Missing the required parameter `si_uuid`")

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.instance_si_uuid_delete_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.instance_si_uuid_delete_with_http_info(**kwargs)  # noqa: E501
            return data

    def instance_si_uuid_delete_with_http_info(self,
                                               **kwargs):  # noqa: E501
        """Delete a service instance  # noqa: E501

        Deletes the specified service instance.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_si_uuid_delete_with_http_info(si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: service instance UUID (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['si_uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}'"
                                " to method instance_si_uuid_delete")
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `instance_si_uuid_delete`"
            )  # noqa: E501
        try:
            return self.client.request('PUT', '/instance/' + kwargs['si_uuid'] + '/delete')
        except ValueError:
            return self.client.request('DELETE', '/service/' + kwargs['si_uuid'])

    def instance_create(self, intent, **kwargs):
        """Create new service instance and/or add new intent

        Creates a new service instance with the given UUID and intent specification
          or add new intent to existing service instance
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_create(body, si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ServiceIntent body: Service instance creation request object. (required)
        :param str si_uuid: Service instance UUID. (required)
        :return: ServiceIntentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid and not kwargs.get('si_uuid'):
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid']:
            raise ValueError("Missing the required parameter `si_uuid`")

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.instance_si_uuid_post_with_http_info(
                intent, **kwargs)  # noqa: E501
        else:
            (data) = self.instance_si_uuid_post_with_http_info(
                intent, **kwargs)  # noqa: E501
            return data

    def instance_si_uuid_post_with_http_info(self, body, **kwargs):
        """Create new service instance and/or add new intent

        Creates a new service instance with the given UUID and intent specification
          or add new intent to existing service instance
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_si_uuid_post_with_http_info(body, si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ServiceIntent body: Service instance creation request object. (required)
        :param str si_uuid: Service instance UUID. (required)
        :return: ServiceIntentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'si_uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}'"
                                 " to method instance_si_uuid_post")
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or params['body'] is None):
            raise ValueError(
                "Missing the required parameter `body` when calling `instance_si_uuid_post`"
            )  # noqa: E501
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `instance_si_uuid_post`"
            )  # noqa: E501

        path_params = {}
        if 'si_uuid' in params:
            path_params['siUUID'] = params['si_uuid']  # noqa: E501

        # proceed is false and sync is true by default

        # Authentication setting
        # auth_settings = ['oAuth2Keycloak']  # noqa: E501
        return self.client.request('POST',
                                   '/instance/' + kwargs['si_uuid'],
                                   body_params=body)

    def instance_modify(self, intent, **kwargs):
        """Create new service instance and/or add new intent

        Creates a new service instance with the given UUID and intent specification
          or add new intent to existing service instance
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_create(body, si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ServiceIntent body: Service instance creation request object. (required)
        :param str si_uuid: Service instance UUID. (required)
        :return: ServiceIntentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid and not kwargs.get('si_uuid'):
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid']:
            raise ValueError("Missing the required parameter `si_uuid`")

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.instance_modify_si_uuid_post_with_http_info(
                intent, **kwargs)  # noqa: E501
        else:
            (data) = self.instance_modify_si_uuid_post_with_http_info(
                intent, **kwargs)  # noqa: E501
            return data

    def instance_modify_si_uuid_post_with_http_info(self, body, **kwargs):
        """Create new service instance and/or add new intent

        Creates a new service instance with the given UUID and intent specification
          or add new intent to existing service instance
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_modify_si_uuid_post_with_http_info(body, si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ServiceIntent body: Service instance creation request object. (required)
        :param str si_uuid: Service instance UUID. (required)
        :return: ServiceIntentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'si_uuid', 'sync', 'proceed']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}'"
                                " to method instance_modify_si_uuid_post")
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or params['body'] is None):
            raise ValueError(
                "Missing the required parameter `body` when calling `instance_modify_si_uuid_post`"
            )  # noqa: E501
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `instance_modify_si_uuid_post`"
            )  # noqa: E501

        path_params = {}
        if 'si_uuid' in params:
            path_params['siUUID'] = params['si_uuid']  # noqa: E501

        query_params = []
        if 'sync' in params:
            query_params.append(('sync', params['sync']))
        if 'proceed' in params:
            query_params.append(('proceed', params['proceed']))

        # Authentication setting
        # auth_settings = ['oAuth2Keycloak']  # noqa: E501
        return self.client.request('POST',
                                   '/instance/' + kwargs['si_uuid'] + '/modify',
                                   body_params=body, query_params=query_params)

    def instance_get_status(self, **kwargs):  # noqa: E501
        """Get instance status  # noqa: E501
        Retrieves the full instance status.  # noqa: E501
        This method makes a synchronous HTTP request by default.
        :param async_req bool
        :param str si_uuid: Service instance UUID. (required)
        :param str verbose: Verbose flag
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid and not kwargs.get('si_uuid'):
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs.get('si_uuid'):
            raise ValueError("Missing the required parameter `si_uuid`")

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.instance_si_uuid_status_get_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.instance_si_uuid_status_get_with_http_info(
                **kwargs)  # noqa: E501
            return data

    def instance_si_uuid_status_get_with_http_info(self, **kwargs):  # noqa: E501
        """Get instance status  # noqa: E501
        Retrieves the full instance status.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_si_uuid_status_get_with_http_info(si_uuid, async_req=True)
        >>> result = thread.get()
        :param async_req bool
        :param str si_uuid: Service instance UUID. (required)
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['si_uuid', 'verbose', 'status', 'async_req', '_return_http_data_only',
                      '_preload_content', '_request_timeout']  # noqa: E501

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}'"
                                " to method instance_si_uuid_status_get")
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'si_uuid' is set
        if 'si_uuid' not in params or params['si_uuid'] is None:
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `instance_si_uuid_status_get`"
            )  # noqa: E501

        if kwargs.get("verbose"):
            return self.client.request('GET', f'/instance/{kwargs.get("si_uuid")}')
        elif kwargs.get("status") == "phase" or kwargs.get("status") == "superstate":
            ret = self.client.request('GET', f'/instance/{kwargs.get("si_uuid")}/status/phase')
        elif kwargs.get("status") == "substatus" or kwargs.get("status") == "substate":
            ret = self.client.request('GET', f'/instance/{kwargs.get("si_uuid")}/substatus')
        elif kwargs.get("status") == "configuration" or kwargs.get("status") == "configstate":
            ret = self.client.request('GET', f'/instance/{kwargs.get("si_uuid")}/status/configuration')
        else:
            ret = self.client.request('GET', f'/instance/{kwargs.get("si_uuid")}/status')
        return ret.replace('"', '')

    def instance_get_intents(self, **kwargs):  # noqa: E501
        """Retrieve intents by service instance  # noqa: E501

        Queries all service intents belonging to given instance UUID.  # noqa: E501
        This method makes a synchronous HTTP request by default.
        :param async_req bool
        :param str si_uuid: Intent UUID. (required)
        :return: list[IntentExpanded]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid and not kwargs.get('si_uuid'):
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid']:
            raise ValueError("Missing the required parameter `si_uuid`")

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.intent_instance_si_uuid_get_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.intent_instance_si_uuid_get_with_http_info(**kwargs)  # noqa: E501
            return data

    def intent_instance_si_uuid_get_with_http_info(self,
                                                   **kwargs):  # noqa: E501
        """Retrieve intents by service instance  # noqa: E501

        Queries all service intents belonging to given instance UUID.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.intent_instance_si_uuid_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: Intent UUID. (required)
        :return: list[IntentExpanded]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['si_uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}'"
                                " to method intent_instance_si_uuid_get")
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `intent_instance_si_uuid_get`"
            )  # noqa: E501

        return self.client.request('GET', '/intent/instance/' + kwargs['si_uuid'])

    def instance_archive(self, **kwargs):  # noqa: E501
        """Archive a service instance  # noqa: E501

        Archive the specified service instance.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_archive(si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: service instance UUID (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid:
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid'] :
            raise ValueError("Missing the required parameter `si_uuid`")

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.instance_si_uuid_archive_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.instance_si_uuid_archive_with_http_info(**kwargs)  # noqa: E501
            return data

    def instance_si_uuid_archive_with_http_info(self, **kwargs):  # noqa: E501
        """Archive a service instance  # noqa: E501

        Archives the specified service instance.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_si_uuid_archive_with_http_info(si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: service instance UUID (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        all_params = ['si_uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method instance_si_uuid_archive" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `instance_si_uuid_archive`"
            )  # noqa: E501

        return self.client.request('PUT', '/instance/' + kwargs['si_uuid'] + '/archive')

    def instance_unarchive(self, **kwargs):  # noqa: E501
        """UnArchive a service instance  # noqa: E501

        UnArchive the specified service instance.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_unarchive(si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: service instance UUID (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid:
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid'] :
            raise ValueError("Missing the required parameter `si_uuid`")

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.instance_si_uuid_unarchive_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.instance_si_uuid_unarchive_with_http_info(**kwargs)  # noqa: E501
            return data

    def instance_si_uuid_unarchive_with_http_info(self, **kwargs):  # noqa: E501
        """Archive a service instance  # noqa: E501

        Archives the specified service instance.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_si_uuid_delete_with_http_info(si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: service instance UUID (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['si_uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method instance_si_uuid_unarchive" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `instance_si_uuid_unarchive`"
            )  # noqa: E501

        return self.client.request('PUT', '/instance/' + kwargs['si_uuid'] + '/unarchive')

    def profile_list(self, **kwargs):
        """Get skimmed profile data  # noqa: E501

        Retrieves the list of profiles the user is permitted to use without any JSON data.  # noqa: E501
        This method makes a synchronous HTTP request by default.
        :param async_req bool
        :return: list[SlimProfile]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.profile_get_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.profile_get_with_http_info(**kwargs)  # noqa: E501
            return data

    def profile_get_with_http_info(self, **kwargs):
        """Get skimmed profile data  # noqa: E501

        Retrieves the list of profiles the user is permitted to use without any JSON data.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: list[SlimProfile]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}'"
                                " to method profile_get")
            params[key] = val
        del params['kwargs']

        return self.client.request('GET', '/profile')

    def profile_describe(self, uuid, **kwargs):  # noqa: E501
        """Get single profile  # noqa: E501

        Retrieves the specified profile.  # noqa: E501
        This method makes a synchronous HTTP request by default.
        :param async_req bool
        :param str uuid: Profile UUID. (required)
        :return: FullProfile
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.profile_uuid_get_with_http_info(uuid,
                                                        **kwargs)  # noqa: E501
        else:
            (data) = self.profile_uuid_get_with_http_info(
                uuid, **kwargs)  # noqa: E501
            return data

    def profile_uuid_get_with_http_info(self, uuid, **kwargs):  # noqa: E501
        """Get single profile  # noqa: E501

        Retrieves the specified profile.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_uuid_get_with_http_info(uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str uuid: Profile UUID. (required)
        :return: FullProfile
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}'"
                                " to method profile_uuid_get")
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'uuid' is set
        if ('uuid' not in params or params['uuid'] is None):
            raise ValueError(
                "Missing the required parameter `uuid` when calling `profile_uuid_get`"
            )  # noqa: E501

        return self.client.request('GET', '/profile/' + uuid)

    def manifest_create(self, template, **kwargs):
        """Create manifest query
        :param async_req bool
        :param ServiceIntent body: Service instance creation request object. (required)
        :param str si_uuid: Service instance UUID. (optional)
        :return: ServiceIntentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid and not kwargs.get('si_uuid'):
            kwargs['si_uuid'] = self.si_uuid

        kwargs['_return_http_data_only'] = True

        if kwargs.get('async_req'):
            return self.instance_si_uuid_manifest_post_with_http_info(
                template, **kwargs)  # noqa: E501
        else:
            (data) = self.instance_si_uuid_manifest_post_with_http_info(
                template, **kwargs)  # noqa: E501
            return data

    def instance_si_uuid_manifest_post_with_http_info(self, body, **kwargs):
        """Create new service instance and/or add new intent

        Creates a new service instance with the given UUID and intent specification
          or add new intent to existing service instance  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_si_uuid_manifest_post_with_http_info(body, si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ServiceIntent body: Service instance creation request object. (required)
        :param str si_uuid: Service instance UUID. (required)
        :return: ServiceIntentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'si_uuid', 'sync']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(
                    f"Got an unexpected keyword argument '{key}'"
                    " to method instance_si_uuid_post_with_http_info")
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or params['body'] is None):
            raise ValueError(
                "Missing the required parameter `body` when calling `instance_modify`"
            )  # noqa: E501

        query_params = []
        if 'sync' in params:
            query_params.append(('sync', params['sync']))
        # sync is true by default

        # Authentication setting
        # auth_settings = ['oAuth2Keycloak']  # noqa: E501
        path_params = {}
        if 'si_uuid' in params:
            path_params['siUUID'] = params['si_uuid']  # noqa: E501
            return self.client.request('POST',
                                   f'/service/manifest/{params["si_uuid"]}/json',
                                   body_params=body,
                                   query_params=query_params,)
        else:
            body_xml = f'<serviceManifest> <serviceUUID></serviceUUID><jsonTemplate>{body}</jsonTemplate></serviceManifest>'
            self.client.config['headers']['Content-type'] = 'application/xml'
            return self.client.request('POST',
                                   f'/service/manifest',
                                   body_params=body_xml,
                                   query_params=query_params,
                                   content_type='xml')




import copy
import logging
from typing import List, Union, Dict

from sense.workflow.base.config import WorkflowConfig
from sense.workflow.base.config_models import ResourceConfig, ProviderConfig
from sense.workflow.base.constants import Constants
from sense.workflow.base.exceptions import ControllerException
from sense.workflow.base.state_models import ResourceState, ProviderState
from sense.workflow.base.utils import get_logger
from sense.workflow.provider.provider import Provider
from sense.workflow.provider.provider_factory import ProviderFactory, default_provider_factory
from sense.workflow.provider.resource_event_listener import ResourceListener


class Controller:
    def __init__(self, *, config: WorkflowConfig, logger: Union[logging.Logger, None] = None):
        import copy

        self.config = copy.deepcopy(config)
        self.logger = logger or get_logger()
        self.provider_factory: Union[ProviderFactory, None] = None
        self.resources: List[ResourceConfig] = []
        self.resource_listener = ControllerResourceListener()

    def init(self, *, session: str,
             provider_factory: ProviderFactory = default_provider_factory,
             provider_states: List[ProviderState]):
        init_provider_map: Dict[str, bool] = dict()

        for provider_config in self.config.get_provider_configs():
            init_provider_map[provider_config.label] = False

        for resource in self.config.get_resource_configs():
            count = resource.attributes.get(Constants.RES_COUNT, 1)
            init_provider_map[resource.provider.label] = init_provider_map[resource.provider.label] or count > 0

        for provider_state in provider_states:
            init_provider_map[provider_state.label] = init_provider_map[provider_state.label] \
                                                      or len(provider_state.states()) > 0

        for provider_config in self.config.get_provider_configs():
            if not init_provider_map[provider_config.label]:
                self.logger.warning(f"Skipping initialization of {provider_config.label}: no resources")
                continue

            name = provider_config.attributes.get('name')
            name = f"{session}-{name}" if name else session
            provider = provider_factory.init_provider(type=provider_config.type,
                                                      label=provider_config.label,
                                                      name=name,
                                                      attributes=provider_config.attributes)
            saved_state = next(filter(lambda s: s.label == provider.label, provider_states), None)
            provider.set_saved_state(saved_state)

        self.resources = self.config.get_resource_configs()

        self.provider_factory = provider_factory
        providers = self.provider_factory.providers
        self.resource_listener.set_providers(providers)

        for provider in providers:
            provider.set_resource_listener(self.resource_listener)

        for resource in self.resources:
            resource_dict = resource.attributes
            resource_dict[Constants.RES_COUNT] = resource_dict.get(Constants.RES_COUNT, 1)
            resource_dict[Constants.RES_NAME_PREFIX] = resource.name
            resource_dict[Constants.CONFIG] = resource_dict.copy()
            resource_dict[Constants.RES_TYPE] = resource.type
            resource_dict[Constants.LABEL] = resource.label
            resource_dict[Constants.EXTERNAL_DEPENDENCIES] = list()
            resource_dict[Constants.RESOLVED_EXTERNAL_DEPENDENCIES] = list()
            resource_dict[Constants.INTERNAL_DEPENDENCIES] = list()
            resource_dict[Constants.RESOLVED_INTERNAL_DEPENDENCIES] = list()
            resource_dict[Constants.SAVED_STATES] = list()

            for dependency in resource.dependencies:
                if dependency.is_external:
                    resource_dict[Constants.EXTERNAL_DEPENDENCIES].append(dependency)
                else:
                    resource_dict[Constants.INTERNAL_DEPENDENCIES].append(dependency)

    def plan(self, provider_states: List[ProviderState]):
        resources = self.resources
        resource_state_map = Controller._build_state_map(provider_states)
        self.logger.info(f"Starting PLAN_PHASE for {len(resources)} resource(s)")
        pf = self.provider_factory
        resources_labels = [r.label for r in resources]
        planned_resources = []

        for resource in resources:
            resource_dict = resource.attributes
            label = resource.provider.label
            provider = pf.get_provider(label=label)
            creation_details = resource_dict[Constants.RES_CREATION_DETAILS] = {}
            creation_details['resources'] = {}
            creation_details['resources'][resource.label] = []
            creation_details['total_count'] = resource_dict[Constants.RES_COUNT]
            creation_details['failed_count'] = 0
            creation_details['created_count'] = 0
            creation_details['in_config_file'] = True
            creation_details['provider_supports_modifiable'] = provider.supports_modify()

            if resource.label in resource_state_map:
                resource_dict = resource.attributes
                resource_dict[Constants.RES_CREATION_DETAILS].update(
                    provider.saved_state.creation_details[resource.label])
                resource_dict[Constants.RES_CREATION_DETAILS]['total_count'] = resource_dict[Constants.RES_COUNT]

            planned_resources.append(resource)

        for state_label, states in resource_state_map.items():
            if state_label not in resources_labels:
                state = states[0]
                resource_dict = state.attributes.copy()
                provider_state = resource_dict.pop(Constants.PROVIDER_STATE)
                provider = pf.get_provider(label=provider_state.label)
                creation_details: Dict = copy.deepcopy(provider_state.creation_details[state_label])
                creation_details['in_config_file'] = False
                creation_details['provider_supports_modifiable'] = provider.supports_modify()
                var_name, _ = tuple(provider_state.label.split('@'))
                provider_config = ProviderConfig(type=provider.type, name=var_name, attrs=provider.config)

                resource_dict = {}
                var_name, _ = tuple(state.label.split('@'))

                if var_name != creation_details['name_prefix']:
                    resource_dict['name'] = creation_details['name_prefix']

                resource_config = ResourceConfig(type=state.type, name=var_name, provider=provider_config,
                                                 attrs=resource_dict)
                resource_dict[Constants.RES_TYPE] = state.type
                resource_dict[Constants.RES_NAME_PREFIX] = creation_details['name_prefix']
                resource_dict[Constants.LABEL] = resource_config.label
                resource_dict[Constants.EXTERNAL_DEPENDENCIES] = list()
                resource_dict[Constants.RESOLVED_EXTERNAL_DEPENDENCIES] = list()
                resource_dict[Constants.INTERNAL_DEPENDENCIES] = list()
                resource_dict[Constants.RESOLVED_INTERNAL_DEPENDENCIES] = list()
                resource_dict[Constants.SAVED_STATES] = list()
                resource_dict[Constants.RES_COUNT] = creation_details['total_count']
                resource_dict[Constants.RES_CREATION_DETAILS] = creation_details
                resource_dict[Constants.CONFIG] = creation_details.pop(Constants.CONFIG)
                planned_resources.append(resource_config)

        self.resources = planned_resources

    def add(self, provider_states: List[ProviderState]):
        resources = self.resources
        self.logger.info(f"Starting ADD_PHASE: Calling ADD ... for {len(resources)} resource(s)")

        exceptions = []
        for resource in resources:
            label = resource.provider.label
            provider = self.provider_factory.get_provider(label=label)

            try:
                provider.validate_resource(resource=resource.attributes)
            except Exception as e:
                exceptions.append(e)
                self.logger.error(e, exc_info=True)

        if exceptions:
            raise ControllerException(exceptions)

        exceptions = []
        resource_state_map = Controller._build_state_map(provider_states)
        for resource in resources:
            label = resource.provider.label
            provider = self.provider_factory.get_provider(label=label)

            if resource.label in resource_state_map:
                resource.attributes[Constants.SAVED_STATES] = resource_state_map[resource.label]

            try:
                provider.add_resource(resource=resource.attributes)
            except Exception as e:
                exceptions.append(e)
                self.logger.error(e, exc_info=True)

        if exceptions:
            raise ControllerException(exceptions)

    def apply(self, provider_states: List[ProviderState]):
        resources = self.resources
        self.logger.info(f"Starting APPLY_PHASE for {len(resources)} resource(s)")
        resource_state_map = Controller._build_state_map(provider_states)
        exceptions = []

        create_and_wait_resource_labels = set()

        for resource in resources:
            resource_dict = resource.attributes

            for dependency in resource_dict[Constants.EXTERNAL_DEPENDENCIES]:
                create_and_wait_resource_labels.add(dependency.resource.label)

            if resource.label in resource_state_map:
                resource.attributes[Constants.SAVED_STATES] = resource_state_map[resource.label]

        for resource in resources:
            if resource.label in create_and_wait_resource_labels:
                provider = self.provider_factory.get_provider(label=resource.provider.label)

                try:
                    provider.create_resource(resource=resource.attributes)
                    provider.wait_for_create_resource(resource=resource.attributes)
                except Exception as e:
                    exceptions.append(e)
                    self.logger.error(e, exc_info=True)

        if exceptions:
            raise ControllerException(exceptions)

        exceptions = []

        for resource in resources:
            if resource.label not in create_and_wait_resource_labels:
                provider = self.provider_factory.get_provider(label=resource.provider.label)

                try:
                    provider.create_resource(resource=resource.attributes)
                except Exception as e:
                    exceptions.append(e)
                    self.logger.error(e, exc_info=True)

        if exceptions:
            raise ControllerException(exceptions)

        exceptions = []

        for resource in resources:
            if resource.label not in create_and_wait_resource_labels:
                provider = self.provider_factory.get_provider(label=resource.provider.label)

                try:
                    provider.wait_for_create_resource(resource=resource.attributes)
                except Exception as e:
                    exceptions.append(e)
                    self.logger.error(e, exc_info=True)

        if exceptions:
            raise ControllerException(exceptions)

    @staticmethod
    def _build_state_map(provider_states: List[ProviderState]) -> Dict[str, List[ResourceState]]:
        resource_state_map = dict()

        for provider_state in provider_states:
            temp_list = provider_state.states()

            for state in temp_list:
                state.attributes[Constants.PROVIDER_STATE] = provider_state

                if state.label in resource_state_map:
                    resource_state_map[state.label].append(state)
                else:
                    resource_state_map[state.label] = [state]

        return resource_state_map

    def destroy(self, *, provider_states: List[ProviderState]):
        exceptions = []
        resource_state_map = Controller._build_state_map(provider_states)
        provider_resource_map = dict()
        failed_resources = []

        for provider_state in provider_states:
            for k in provider_state.failed:
                failed_resources.append(k)

        for provider_state in provider_states:
            key = provider_state.label
            provider_resource_map[key] = list()

        temp = self.resources
        temp.reverse()

        for resource in temp:
            if resource.label in resource_state_map:
                key = resource.provider.label
                external_dependencies = resource.attributes.get(Constants.EXTERNAL_DEPENDENCIES, [])
                external_states = [resource_state_map[ed.resource.label] for ed in external_dependencies]
                resource.attributes[Constants.EXTERNAL_DEPENDENCY_STATES] = sum(external_states, [])
                provider_resource_map[key].append(resource)
                resource.attributes[Constants.SAVED_STATES] = resource_state_map[resource.label]

        remaining_resources = list()
        skip_resources = set()

        for resource in temp:
            if resource.label not in resource_state_map and resource.label not in failed_resources:
                continue

            provider_label = resource.provider.label
            provider = self.provider_factory.get_provider(label=provider_label)
            external_states = resource.attributes.get(Constants.EXTERNAL_DEPENDENCY_STATES, list())

            if resource.label in skip_resources:
                self.logger.warning(f"Skipping deleting resource: {resource} with {provider_label}")
                remaining_resources.append(resource)
                skip_resources.update([external_state.label for external_state in external_states])
                continue

            try:
                provider.delete_resource(resource=resource.attributes)
            except Exception as e:
                self.logger.warning(f"Exception occurred while deleting resource: {e} using {provider_label}",
                                    exc_info=True)
                remaining_resources.append(resource)
                skip_resources.update([external_state.label for external_state in external_states])
                exceptions.append(e)

        if not remaining_resources:
            provider_states.clear()
            return

        provider_states_copy = provider_states.copy()
        provider_states.clear()

        for provider_state in provider_states_copy:
            provider_state.service_states.clear()

            if self.provider_factory.has_provider(label=provider_state.label):
                provider = self.provider_factory.get_provider(label=provider_state.label)
                provider_state.failed = provider.failed

                for remaining_resource in [r for r in remaining_resources if r.provider.label == provider_state.label]:
                    if remaining_resource.label in resource_state_map:
                        resource_states = resource_state_map[remaining_resource.label]
                        provider_state.add_all(resource_states)

                if provider_state.states() or provider_state.failed:
                    provider_states.append(provider_state)

        if exceptions:
            raise ControllerException(exceptions)

    def get_states(self) -> List[ProviderState]:
        provider_states = []

        for provider in self.provider_factory.providers:
            provider_state = provider.get_state()
            provider_states.append(provider_state)

        return provider_states


class ControllerResourceListener(ResourceListener):
    def __init__(self):
        self.providers = list()

    def set_providers(self, providers: list):
        self.providers = providers

    def on_added(self, *, source, provider: Provider, resource: object):
        for temp_provider in self.providers:
            temp_provider.on_added(source=self, provider=provider, resource=resource)

    def on_created(self, *, source, provider: Provider, resource: object):
        for temp_provider in self.providers:
            if temp_provider == provider:
                temp_provider.on_created(source=self, provider=provider, resource=resource)
                break

        for temp_provider in self.providers:
            if temp_provider != provider:
                temp_provider.on_created(source=self, provider=provider, resource=resource)

    def on_deleted(self, *, source, provider: Provider, resource: object):
        for temp_provider in self.providers:
            temp_provider.on_deleted(source=self, provider=provider, resource=resource)



# coding: utf-8

"""
    SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Log(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'timestamp': 'str',
        'reference_uuid': 'str',
        'level': 'LogLevel',
        'logger': 'str',
        'message': 'str',
        'event': 'str',
        'exception': 'str',
        'target_id': 'str'
    }

    attribute_map = {
        'id': 'id',
        'timestamp': 'timestamp',
        'reference_uuid': 'referenceUUID',
        'level': 'level',
        'logger': 'logger',
        'message': 'message',
        'event': 'event',
        'exception': 'exception',
        'target_id': 'targetID'
    }

    def __init__(self, id=None, timestamp=None, reference_uuid=None, level=None, logger=None, message=None, event=None, exception=None, target_id=None):  # noqa: E501
        """Log - a model defined in Swagger"""  # noqa: E501
        self._id = None
        self._timestamp = None
        self._reference_uuid = None
        self._level = None
        self._logger = None
        self._message = None
        self._event = None
        self._exception = None
        self._target_id = None
        self.discriminator = None
        self.id = id
        self.timestamp = timestamp
        if reference_uuid is not None:
            self.reference_uuid = reference_uuid
        self.level = level
        if logger is not None:
            self.logger = logger
        self.message = message
        self.event = event
        if exception is not None:
            self.exception = exception
        if target_id is not None:
            self.target_id = target_id

    @property
    def id(self):
        """Gets the id of this Log.  # noqa: E501

        Database ID for the entry.  # noqa: E501

        :return: The id of this Log.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Log.

        Database ID for the entry.  # noqa: E501

        :param id: The id of this Log.  # noqa: E501
        :type: str
        """
        if id is None:
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def timestamp(self):
        """Gets the timestamp of this Log.  # noqa: E501

        When the log was registered.  # noqa: E501

        :return: The timestamp of this Log.  # noqa: E501
        :rtype: str
        """
        return self._timestamp

    @timestamp.setter
    def timestamp(self, timestamp):
        """Sets the timestamp of this Log.

        When the log was registered.  # noqa: E501

        :param timestamp: The timestamp of this Log.  # noqa: E501
        :type: str
        """
        if timestamp is None:
            raise ValueError("Invalid value for `timestamp`, must not be `None`")  # noqa: E501

        self._timestamp = timestamp

    @property
    def reference_uuid(self):
        """Gets the reference_uuid of this Log.  # noqa: E501

        A service instance UUID that the log is associated with.  # noqa: E501

        :return: The reference_uuid of this Log.  # noqa: E501
        :rtype: str
        """
        return self._reference_uuid

    @reference_uuid.setter
    def reference_uuid(self, reference_uuid):
        """Sets the reference_uuid of this Log.

        A service instance UUID that the log is associated with.  # noqa: E501

        :param reference_uuid: The reference_uuid of this Log.  # noqa: E501
        :type: str
        """

        self._reference_uuid = reference_uuid

    @property
    def level(self):
        """Gets the level of this Log.  # noqa: E501


        :return: The level of this Log.  # noqa: E501
        :rtype: LogLevel
        """
        return self._level

    @level.setter
    def level(self, level):
        """Sets the level of this Log.


        :param level: The level of this Log.  # noqa: E501
        :type: LogLevel
        """
        if level is None:
            raise ValueError("Invalid value for `level`, must not be `None`")  # noqa: E501

        self._level = level

    @property
    def logger(self):
        """Gets the logger of this Log.  # noqa: E501

        Class name of the logger that created this entry.  # noqa: E501

        :return: The logger of this Log.  # noqa: E501
        :rtype: str
        """
        return self._logger

    @logger.setter
    def logger(self, logger):
        """Sets the logger of this Log.

        Class name of the logger that created this entry.  # noqa: E501

        :param logger: The logger of this Log.  # noqa: E501
        :type: str
        """

        self._logger = logger

    @property
    def message(self):
        """Gets the message of this Log.  # noqa: E501

        The main event data.  # noqa: E501

        :return: The message of this Log.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this Log.

        The main event data.  # noqa: E501

        :param message: The message of this Log.  # noqa: E501
        :type: str
        """
        if message is None:
            raise ValueError("Invalid value for `message`, must not be `None`")  # noqa: E501

        self._message = message

    @property
    def event(self):
        """Gets the event of this Log.  # noqa: E501

        The specific method or signature of that created this entry.  # noqa: E501

        :return: The event of this Log.  # noqa: E501
        :rtype: str
        """
        return self._event

    @event.setter
    def event(self, event):
        """Sets the event of this Log.

        The specific method or signature of that created this entry.  # noqa: E501

        :param event: The event of this Log.  # noqa: E501
        :type: str
        """
        if event is None:
            raise ValueError("Invalid value for `event`, must not be `None`")  # noqa: E501

        self._event = event

    @property
    def exception(self):
        """Gets the exception of this Log.  # noqa: E501

        An exception string, including trace and formatted information.  # noqa: E501

        :return: The exception of this Log.  # noqa: E501
        :rtype: str
        """
        return self._exception

    @exception.setter
    def exception(self, exception):
        """Sets the exception of this Log.

        An exception string, including trace and formatted information.  # noqa: E501

        :param exception: The exception of this Log.  # noqa: E501
        :type: str
        """

        self._exception = exception

    @property
    def target_id(self):
        """Gets the target_id of this Log.  # noqa: E501

        Extra UUID field, for additional tracking or metadata purposes. Specific to the log event.  # noqa: E501

        :return: The target_id of this Log.  # noqa: E501
        :rtype: str
        """
        return self._target_id

    @target_id.setter
    def target_id(self, target_id):
        """Sets the target_id of this Log.

        Extra UUID field, for additional tracking or metadata purposes. Specific to the log event.  # noqa: E501

        :param target_id: The target_id of this Log.  # noqa: E501
        :type: str
        """

        self._target_id = target_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Log, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Log):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import os
import unittest
import pytest
import re
import json
import string
import random
import sys

from sense.client.workflow_combined_api import WorkflowCombinedApi

# Append root for proper imports
sys.path.append('')
#

#
from sense.client.intent_api import IntentApi
from sense.client.instance_api import InstanceApi
from sense.common import loadJSON


#


class TestManifestApi(unittest.TestCase):
    def setUp(self) -> None:
        self.client = WorkflowCombinedApi()

    def test_manifest_create(self):
        #
        # TESTING: - /service/manifest/{siUUID}
        template = loadJSON("requests/manifest-2.json")
        self.client.si_uuid = "9d327176-1516-4061-a0d0-660ae866d141"
        res = self.client.manifest_create(json.dumps(template))
        print()
        print(res)
        assert res

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# coding: utf-8
"""
    SENSE-O Northbound Metadata API
"""
from sense.client.requestwrapper import RequestWrapper
from sense.common import classwrapper


@classwrapper
class MetadataApi():
    """Metadata API"""
    def __init__(self, req_wrapper=None):
        if req_wrapper is None:
            self.client = RequestWrapper()
        else:
            self.client = req_wrapper
        if 'SI_UUID' in self.client.config:
            self.si_uuid = self.client.config['SI_UUID']
        else:
            self.si_uuid = None

    def get_metadata(self, **kwargs):
        """Retrieve metadata by domain and name"""
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_metadata_with_http_info(**kwargs)  # noqa: E501
        (data) = self.get_metadata_with_http_info(**kwargs)  # noqa: E501
        return data

    def get_metadata_with_http_info(self, **kwargs):
        """Retrieve metadata by domain and name call"""
        all_params = ['async_req', '_return_http_data_only', '_preload_content', '_request_timeout', 'domain', 'name']
        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}' to method get_metadata_with_http_info")
            params[key] = val
        del params['kwargs']

        return self.client.request('GET', f'/meta/{kwargs["domain"]}/{kwargs["name"]}')

    def post_metadata(self, data, **kwargs):
        """Create metadata with domain and name"""
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.post_metadata_with_http_info(data, **kwargs)  # noqa: E501
        (data) = self.post_metadata_with_http_info(data, **kwargs)  # noqa: E501
        return data

    def post_metadata_with_http_info(self, data, **kwargs):
        """Create metadata with domain and name call"""
        all_params = ['async_req', '_return_http_data_only', '_preload_content', '_request_timeout', 'domain', 'name']
        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}' to method post_metadata_with_http_info")
            params[key] = val
        del params['kwargs']

        return self.client.request('POST', f'/meta/{kwargs["domain"]}/{kwargs["name"]}', body_params=data)

    def update_metadata(self, data, **kwargs):
        """Update metadata with domain and name"""
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.update_metadata_with_http_info(data, **kwargs)  # noqa: E501
        (data) = self.update_metadata_with_http_info(data, **kwargs)  # noqa: E501
        return data

    def update_metadata_with_http_info(self, data, **kwargs):
        """Update metadata with domain and name call"""
        all_params = ['async_req', '_return_http_data_only', '_preload_content', '_request_timeout', 'domain', 'name']
        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}' to method update_metadata_with_http_info")
            params[key] = val
        del params['kwargs']

        return self.client.request('PUT', f'/meta/{kwargs["domain"]}/{kwargs["name"]}/update', body_params=data)

    def delete_metadata(self, **kwargs):
        """Delete metadata by domain and name"""
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_metadata_with_http_info(**kwargs)  # noqa: E501
        (data) = self.delete_metadata_with_http_info(**kwargs)  # noqa: E501
        return data

    def delete_metadata_with_http_info(self, **kwargs):
        """Delete metadata by domain and name call"""
        all_params = ['async_req', '_return_http_data_only', '_preload_content', '_request_timeout', 'domain', 'name']
        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}' to method delete_metadata_with_http_info")
            params[key] = val
        del params['kwargs']

        return self.client.request('DELETE', f'/meta/{kwargs["domain"]}/{kwargs["name"]}')

    def get_metadata_policies(self, **kwargs):
        """Retrieve metadata policies by domain and name"""
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_metadata_policies_with_http_info(**kwargs)  # noqa: E501
        (data) = self.get_metadata_policies_with_http_info(**kwargs)  # noqa: E501
        return data

    def get_metadata_policies_with_http_info(self, **kwargs):
        """Retrieve metadata policies by domain and name call"""
        all_params = ['async_req', '_return_http_data_only', '_preload_content', '_request_timeout', 'domain', 'name']
        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}' to method get_metadata_policies_with_http_info")
            params[key] = val
        del params['kwargs']

        return self.client.request('GET', f'/meta/{kwargs["domain"]}/{kwargs["name"]}/policy')

    def update_metadata_policy(self, data, **kwargs):
        """Update metadata policy by domain and name"""
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.update_metadata_policy_with_http_info(data, **kwargs)  # noqa: E501
        (data) = self.update_metadata_policy_with_http_info(data, **kwargs)  # noqa: E501
        return data

    def update_metadata_policy_with_http_info(self, data, **kwargs):
        """Update metadata policy by domain and name call"""
        all_params = ['async_req', '_return_http_data_only', '_preload_content', '_request_timeout', 'domain', 'name']
        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}' to method update_metadata_policy_with_http_info")
            params[key] = val
        del params['kwargs']

        return self.client.request('POST', f'/meta/{kwargs["domain"]}/{kwargs["name"]}/policy', body_params=data)

    def delete_metadata_policy(self, **kwargs):
        """Delete metadata policy by domain and name"""
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_metadata_policy_with_http_info(**kwargs)  # noqa: E501
        (data) = self.delete_metadata_policy_with_http_info(**kwargs)  # noqa: E501
        return data

    def delete_metadata_policy_with_http_info(self, **kwargs):
        """Delete metadata policy by domain and name call"""
        all_params = ['async_req', '_return_http_data_only', '_preload_content', '_request_timeout', 'domain', 'name',
                      'policy']
        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(f"Got an unexpected keyword argument '{key}' to method delete_metadata_policy_with_http_info")
            params[key] = val
        del params['kwargs']

        return self.client.request('DELETE',
                                   f'/meta/{kwargs["domain"]}/{kwargs["name"]}/policy/name/{kwargs["policy"]}')
